@@ -52,16 +52,17 @@ format_check (GdkPixbufModule *module, guchar *buffer, int size)
 	gchar m;
 	GdkPixbufModulePattern *pattern;
 	gboolean anchored;
-	guchar *prefix, *mask;
+	guchar *prefix;
+	gchar *mask;
 
 	for (pattern = module->info->signature; pattern->prefix; pattern++) {
 		if (pattern->mask && pattern->mask[0] == '*') {
-			prefix = pattern->prefix + 1;
+			prefix = (guchar *)pattern->prefix + 1;
 			mask = pattern->mask + 1;
 			anchored = FALSE;
 		}
 		else {
-			prefix = pattern->prefix;
+			prefix = (guchar *)pattern->prefix;
 			mask = pattern->mask;
 			anchored = TRUE;
 		}
@@ -453,7 +454,7 @@ _gdk_pixbuf_load_module_unlocked (GdkPixbufModule *image_module,
         g_return_val_if_fail (image_module->module == NULL, FALSE);
 
 	path = image_module->module_path;
-	module = g_module_open (path, G_MODULE_BIND_LAZY);
+	module = g_module_open (path, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
 
         if (!module) {
                 g_set_error (error,
@@ -839,7 +840,7 @@ gdk_pixbuf_new_from_file (const char *filename,
 	GdkPixbuf *pixbuf;
 	int size;
 	FILE *f;
-	guchar buffer[256];
+	guchar buffer[1024];
 	GdkPixbufModule *image_module;
 	gchar *display_name;
 
@@ -850,12 +851,13 @@ gdk_pixbuf_new_from_file (const char *filename,
 
 	f = g_fopen (filename, "rb");
 	if (!f) {
+		gint save_errno = errno;
                 g_set_error (error,
                              G_FILE_ERROR,
-                             g_file_error_from_errno (errno),
+                             g_file_error_from_errno (save_errno),
                              _("Failed to open file '%s': %s"),
                              display_name,
-                             g_strerror (errno));
+                             g_strerror (save_errno));
                 g_free (display_name);
 		return NULL;
         }
@@ -1100,13 +1102,14 @@ gdk_pixbuf_new_from_file_at_scale (const char *filename,
 
 	f = g_fopen (filename, "rb");
 	if (!f) {
+		gint save_errno = errno;
                 gchar *display_name = g_filename_display_name (filename);
                 g_set_error (error,
                              G_FILE_ERROR,
-                             g_file_error_from_errno (errno),
+                             g_file_error_from_errno (save_errno),
                              _("Failed to open file '%s': %s"),
                              display_name,
-                             g_strerror (errno));
+                             g_strerror (save_errno));
                 g_free (display_name);
 		return NULL;
         }
@@ -1369,11 +1372,12 @@ save_to_file_callback (const gchar *buf,
 
 	n = fwrite (buf, 1, count, filehandle);
 	if (n != count) {
+		gint save_errno = errno;
                 g_set_error (error,
                              G_FILE_ERROR,
-                             g_file_error_from_errno (errno),
+                             g_file_error_from_errno (save_errno),
                              _("Error writing to image file: %s"),
-                             g_strerror (errno));
+                             g_strerror (save_errno));
                 return FALSE;
 	}
 	return TRUE;
@@ -1461,9 +1465,10 @@ save_to_callback_with_tmp_file (GdkPixbufModule   *image_module,
 		goto end;
 	f = fdopen (fd, "wb+");
 	if (f == NULL) {
+		gint save_errno = errno;
 		g_set_error (error,
 			     G_FILE_ERROR,
-			     g_file_error_from_errno (errno),
+			     g_file_error_from_errno (save_errno),
 			     _("Failed to open temporary file"));
 		goto end;
 	}
@@ -1486,9 +1491,10 @@ save_to_callback_with_tmp_file (GdkPixbufModule   *image_module,
 			break;
 	}
 	if (ferror (f)) {
+		gint save_errno = errno;
 		g_set_error (error,
 			     G_FILE_ERROR,
-			     g_file_error_from_errno (errno),
+			     g_file_error_from_errno (save_errno),
 			     _("Failed to read from temporary file"));
 		goto end;
 	}
@@ -1694,7 +1700,7 @@ gdk_pixbuf_save (GdkPixbuf  *pixbuf,
  * @option_values: values for named options
  * @error: return location for error, or %NULL
  *
- * Saves pixbuf to a file in @type, which is currently "jpeg", "png", "ico" or "bmp".
+ * Saves pixbuf to a file in @type, which is currently "jpeg", "png", "tiff", "ico" or "bmp".
  * If @error is set, %FALSE will be returned. 
  * See gdk_pixbuf_save () for more details.
  *
@@ -1719,13 +1725,14 @@ gdk_pixbuf_savev (GdkPixbuf  *pixbuf,
         f = g_fopen (filename, "wb");
         
         if (f == NULL) {
+		gint save_errno = errno;
                 gchar *display_name = g_filename_display_name (filename);
                 g_set_error (error,
                              G_FILE_ERROR,
-                             g_file_error_from_errno (errno),
+                             g_file_error_from_errno (save_errno),
                              _("Failed to open '%s' for writing: %s"),
                              display_name,
-                             g_strerror (errno));
+                             g_strerror (save_errno));
                 g_free (display_name);
                 return FALSE;
         }
@@ -1743,13 +1750,14 @@ gdk_pixbuf_savev (GdkPixbuf  *pixbuf,
        }
 
        if (fclose (f) < 0) {
+	       gint save_errno = errno;
                gchar *display_name = g_filename_display_name (filename);
                g_set_error (error,
                             G_FILE_ERROR,
-                            g_file_error_from_errno (errno),
+                            g_file_error_from_errno (save_errno),
                             _("Failed to close '%s' while writing image, all data may not have been saved: %s"),
                             display_name,
-                            g_strerror (errno));
+                            g_strerror (save_errno));
                g_free (display_name);
                return FALSE;
        }
@@ -1855,7 +1863,7 @@ gdk_pixbuf_save_to_callback    (GdkPixbuf  *pixbuf,
  * @error: return location for error, or %NULL
  *
  * Saves pixbuf to a callback in format @type, which is currently "jpeg",
- * "png", "ico" or "bmp".  If @error is set, %FALSE will be returned. See
+ * "png", "tiff", "ico" or "bmp".  If @error is set, %FALSE will be returned. See
  * gdk_pixbuf_save_to_callback () for more details.
  *
  * Return value: whether an error was set
@@ -1901,7 +1909,7 @@ gdk_pixbuf_save_to_callbackv   (GdkPixbuf  *pixbuf,
  * @Varargs: list of key-value save options
  *
  * Saves pixbuf to a new buffer in format @type, which is currently "jpeg",
- * "png", "ico" or "bmp".  This is a convenience function that uses
+ * "png", "tiff", "ico" or "bmp".  This is a convenience function that uses
  * gdk_pixbuf_save_to_callback() to do the real work. Note that the buffer 
  * is not nul-terminated and may contain embedded  nuls.
  * If @error is set, %FALSE will be returned and @string will be set to
@@ -1989,7 +1997,7 @@ save_to_buffer_callback (const gchar *data,
  * @error: return location for error, or %NULL
  *
  * Saves pixbuf to a new buffer in format @type, which is currently "jpeg",
- * "png", "ico" or "bmp".  See gdk_pixbuf_save_to_buffer() for more details.
+ * "tiff", "png", "ico" or "bmp".  See gdk_pixbuf_save_to_buffer() for more details.
  *
  * Return value: whether an error was set
  *
@@ -2200,10 +2208,10 @@ gdk_pixbuf_format_set_disabled (GdkPixbufFormat *format,
  * gdk_pixbuf_format_get_license:
  * @format: a #GdkPixbufFormat
  *
- * Returns information about the license of the image loader
- * for the format. The returned string should be a shorthand for 
- * a wellknown license, e.g. "LGPL", "GPL", "QPL", "GPL/QPL",
- * or "other" to indicate some other license.  
+ * Returns information about the license of the image loader for the format. The
+ * returned string should be a shorthand for a wellknown license, e.g. "LGPL",
+ * "GPL", "QPL", "GPL/QPL", or "other" to indicate some other license.  This
+ * string should be freed with g_free() when it's no longer needed.
  *
  * Returns: a string describing the license of @format. 
  *
