@@ -30,8 +30,7 @@
 #include "gdkscreen.h"
 #include "gdkalias.h"
 
-/* This is for P_() ... a bit non-kosher, but works fine */
-#include "gtk/gtkintl.h"
+#include "gdkintl.h"
 
 #define GDK_INFO_KEY "gdk-info"
 
@@ -58,6 +57,8 @@ struct _GdkPangoRendererPrivate
   /* Current target */
   GdkDrawable *drawable;
   GdkGC *base_gc;
+
+  gboolean gc_changed;
 };
 
 static PangoAttrType gdk_pango_attr_stipple_type;
@@ -131,6 +132,21 @@ emboss_context (cairo_t *cr)
   cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 }
 
+static inline gboolean
+color_equal (PangoColor *c1, PangoColor *c2)
+{
+  if (!c1 && !c2)
+    return TRUE;
+
+  if (c1 && c2 &&
+      c1->red == c2->red &&
+      c1->green == c2->green &&
+      c1->blue == c2->blue)
+    return TRUE;
+
+  return FALSE;
+}
+
 static cairo_t *
 get_cairo_context (GdkPangoRenderer *gdk_renderer,
 		   PangoRenderPart   part)
@@ -156,28 +172,46 @@ get_cairo_context (GdkPangoRenderer *gdk_renderer,
 	  cairo_set_matrix (priv->cr, &cairo_matrix);
 	}
     }
-  
-  priv->last_part = (PangoRenderPart)-1;
+
   if (part != priv->last_part)
     {
-      PangoColor *pango_color = pango_renderer_get_color (renderer,
-							  part);
-      GdkColor *color = NULL;
+      PangoColor *pango_color;
+      GdkColor *color;
       GdkColor tmp_color;
-      if (pango_color)
+      gboolean changed;
+
+      pango_color = pango_renderer_get_color (renderer, part);
+      
+      if (priv->last_part != -1)
+	changed = priv->gc_changed ||
+	  priv->stipple[priv->last_part] != priv->stipple[part] ||
+	  !color_equal (pango_color,
+			pango_renderer_get_color (renderer, priv->last_part));
+      else
+	changed = TRUE;
+      
+      if (changed)
 	{
-	  tmp_color.red = pango_color->red;
-	  tmp_color.green = pango_color->green;
-	  tmp_color.blue = pango_color->blue;
-	  
-	  color = &tmp_color;
+	  if (pango_color)
+	    {
+	      tmp_color.red = pango_color->red;
+	      tmp_color.green = pango_color->green;
+	      tmp_color.blue = pango_color->blue;
+	      
+	      color = &tmp_color;
+	    }
+	  else
+	    color = NULL;
+
+	  _gdk_gc_update_context (priv->base_gc,
+				  priv->cr,
+				  color,
+				  priv->stipple[part],
+				  priv->gc_changed);
 	}
 
-      _gdk_gc_update_context (priv->base_gc,
-			      priv->cr,
-			      color,
-			      priv->stipple[part]);
       priv->last_part = part;
+      priv->gc_changed = FALSE;
     }
 
   return priv->cr;
@@ -201,12 +235,12 @@ gdk_pango_renderer_draw_glyphs (PangoRenderer    *renderer,
     {
       cairo_save (cr);
       emboss_context (cr);
-      cairo_move_to (cr, x / PANGO_SCALE, y / PANGO_SCALE);
+      cairo_move_to (cr, (double)x / PANGO_SCALE, (double)y / PANGO_SCALE);
       pango_cairo_show_glyph_string (cr, font, glyphs);
       cairo_restore (cr);
     }
   
-  cairo_move_to (cr, x / PANGO_SCALE, y / PANGO_SCALE);
+  cairo_move_to (cr, (double)x / PANGO_SCALE, (double)y / PANGO_SCALE);
   pango_cairo_show_glyph_string (cr, font, glyphs);
 }
 
@@ -230,6 +264,7 @@ gdk_pango_renderer_draw_glyphs (PangoRenderer    *renderer,
  */
 #define HEIGHT_SQUARES 2.5
 
+/* Cut-and-pasted between here and pango/pango/pangocairo-render.c */
 static void
 draw_error_underline (cairo_t *cr,
 		      double  x,
@@ -482,6 +517,7 @@ gdk_pango_renderer_init (GdkPangoRenderer *renderer)
 						GdkPangoRendererPrivate);
 
   renderer->priv->last_part = (PangoRenderPart)-1;
+  renderer->priv->gc_changed = TRUE;
 }
 
 static void
@@ -541,12 +577,14 @@ gdk_pango_renderer_new (GdkScreen *screen)
 
 static void
 on_renderer_display_closed (GdkDisplay       *display,
+                            gboolean          is_error,
 			    GdkPangoRenderer *renderer)
 {
-  g_signal_handlers_disconnect_by_func (renderer->priv->screen,
-					(gpointer)on_renderer_display_closed,
+  g_signal_handlers_disconnect_by_func (display,
+					on_renderer_display_closed,
 					renderer);
-  g_object_set_data (G_OBJECT (renderer->priv->screen), "gdk-pango-renderer", NULL);
+  g_object_set_data (G_OBJECT (renderer->priv->screen),
+                     g_intern_static_string ("gdk-pango-renderer"), NULL);
 }
 
 /**
@@ -579,7 +617,8 @@ gdk_pango_renderer_get_default (GdkScreen *screen)
   if (!renderer)
     {
       renderer = gdk_pango_renderer_new (screen);
-      g_object_set_data_full (G_OBJECT (screen), "gdk-pango-renderer", renderer,
+      g_object_set_data_full (G_OBJECT (screen), 
+                              g_intern_static_string ("gdk-pango-renderer"), renderer,
 			      (GDestroyNotify)g_object_unref);
 
       g_signal_connect (gdk_screen_get_display (screen), "closed",
@@ -650,6 +689,8 @@ gdk_pango_renderer_set_gc (GdkPangoRenderer *gdk_renderer,
       priv->base_gc = gc;
       if (priv->base_gc)
 	g_object_ref (priv->base_gc);
+
+      priv->gc_changed = TRUE;
     }
 }
 
@@ -1383,10 +1424,10 @@ gdk_pango_context_get_for_screen (GdkScreen *screen)
   
   context = pango_cairo_font_map_create_context (PANGO_CAIRO_FONT_MAP (fontmap));
 
-  options = gdk_screen_get_font_options_libgtk_only (screen);
+  options = gdk_screen_get_font_options (screen);
   pango_cairo_context_set_font_options (context, options);
 
-  dpi = gdk_screen_get_resolution_libgtk_only (screen);
+  dpi = gdk_screen_get_resolution (screen);
   pango_cairo_context_set_resolution (context, dpi);
 
   return context;
