@@ -25,9 +25,10 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkbox.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 enum {
   PROP_0,
@@ -44,8 +45,6 @@ enum {
   CHILD_PROP_POSITION
 };
 
-static void gtk_box_class_init (GtkBoxClass    *klass);
-static void gtk_box_init       (GtkBox         *box);
 static void gtk_box_set_property (GObject         *object,
 				  guint            prop_id,
 				  const GValue    *value,
@@ -75,36 +74,7 @@ static void gtk_box_get_child_property (GtkContainer    *container,
 static GType gtk_box_child_type (GtkContainer   *container);
      
 
-static GtkContainerClass *parent_class = NULL;
-
-
-GType
-gtk_box_get_type (void)
-{
-  static GType box_type = 0;
-
-  if (!box_type)
-    {
-      static const GTypeInfo box_info =
-      {
-	sizeof (GtkBoxClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_box_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkBox),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_box_init,
-	NULL,		/* value_table */
-      };
-
-      box_type = g_type_register_static (GTK_TYPE_CONTAINER, "GtkBox", 
-					 &box_info, G_TYPE_FLAG_ABSTRACT);
-    }
-
-  return box_type;
-}
+G_DEFINE_ABSTRACT_TYPE (GtkBox, gtk_box, GTK_TYPE_CONTAINER);
 
 static void
 gtk_box_class_init (GtkBoxClass *class)
@@ -112,8 +82,6 @@ gtk_box_class_init (GtkBoxClass *class)
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
   GtkContainerClass *container_class = GTK_CONTAINER_CLASS (class);
 
-  parent_class = g_type_class_peek_parent (class);
-
   gobject_class->set_property = gtk_box_set_property;
   gobject_class->get_property = gtk_box_get_property;
    
@@ -132,7 +100,7 @@ gtk_box_class_init (GtkBoxClass *class)
                                                      0,
                                                      G_MAXINT,
                                                      0,
-                                                     G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                     GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_HOMOGENEOUS,
@@ -140,7 +108,7 @@ gtk_box_class_init (GtkBoxClass *class)
 							 P_("Homogeneous"),
 							 P_("Whether the children should all be the same size"),
 							 FALSE,
-							 G_PARAM_READABLE | G_PARAM_WRITABLE));
+							 GTK_PARAM_READWRITE));
 
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_EXPAND,
@@ -148,35 +116,35 @@ gtk_box_class_init (GtkBoxClass *class)
 								    P_("Expand"), 
 								    P_("Whether the child should receive extra space when the parent grows"),
 								    TRUE,
-								    G_PARAM_READWRITE));
+								    GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_FILL,
 					      g_param_spec_boolean ("fill", 
 								    P_("Fill"), 
 								    P_("Whether extra space given to the child should be allocated to the child or used as padding"),
 								    TRUE,
-								    G_PARAM_READWRITE));
+								    GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_PADDING,
 					      g_param_spec_uint ("padding", 
 								 P_("Padding"), 
 								 P_("Extra space to put between the child and its neighbors, in pixels"),
 								 0, G_MAXINT, 0,
-								 G_PARAM_READWRITE));
+								 GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_PACK_TYPE,
-					      g_param_spec_enum ("pack_type", 
+					      g_param_spec_enum ("pack-type", 
 								 P_("Pack type"), 
 								 P_("A GtkPackType indicating whether the child is packed with reference to the start or end of the parent"),
 								 GTK_TYPE_PACK_TYPE, GTK_PACK_START,
-								 G_PARAM_READWRITE));
+								 GTK_PARAM_READWRITE));
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_POSITION,
 					      g_param_spec_int ("position", 
 								P_("Position"), 
 								P_("The index of the child in the parent"),
 								-1, G_MAXINT, 0,
-								G_PARAM_READWRITE));
+								GTK_PARAM_READWRITE));
 }
 
 static void
@@ -391,7 +359,7 @@ gtk_box_pack_start (GtkBox    *box,
   gtk_widget_child_notify (child, "expand");
   gtk_widget_child_notify (child, "fill");
   gtk_widget_child_notify (child, "padding");
-  gtk_widget_child_notify (child, "pack_type");
+  gtk_widget_child_notify (child, "pack-type");
   gtk_widget_child_notify (child, "position");
   gtk_widget_thaw_child_notify (child);
 }
@@ -426,7 +394,7 @@ gtk_box_pack_end (GtkBox    *box,
   gtk_widget_child_notify (child, "expand");
   gtk_widget_child_notify (child, "fill");
   gtk_widget_child_notify (child, "padding");
-  gtk_widget_child_notify (child, "pack_type");
+  gtk_widget_child_notify (child, "pack-type");
   gtk_widget_child_notify (child, "position");
   gtk_widget_thaw_child_notify (child);
 }
@@ -624,7 +592,7 @@ gtk_box_set_child_packing (GtkBox               *box,
 	child_info->pack = GTK_PACK_END;
       else
 	child_info->pack = GTK_PACK_START;
-      gtk_widget_child_notify (child, "pack_type");
+      gtk_widget_child_notify (child, "pack-type");
 
       if (GTK_WIDGET_VISIBLE (child) && GTK_WIDGET_VISIBLE (box))
 	gtk_widget_queue_resize (child);
@@ -712,3 +680,6 @@ gtk_box_forall (GtkContainer *container,
 	(* callback) (child->widget, callback_data);
     }
 }
+
+#define __GTK_BOX_C__
+#include "gtkaliasdef.c"
