@@ -35,13 +35,13 @@
 #include "gtkhbox.h"
 #include "gtkstock.h"
 #include "gtkiconfactory.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkalias.h"
 
-#define CHILD_SPACING     1
-
 static const GtkBorder default_default_border = { 1, 1, 1, 1 };
 static const GtkBorder default_default_outside_border = { 0, 0, 0, 0 };
+static const GtkBorder default_inner_border = { 1, 1, 1, 1 };
 
 /* Time out before giving up on getting a key release when animating
  * the close button.
@@ -80,50 +80,52 @@ struct _GtkButtonPrivate
   GtkWidget   *image;
   guint        align_set : 1;
   guint        image_is_stock : 1;
+  guint        has_grab : 1;
+  guint32      grab_time;
 };
 
-static void gtk_button_class_init     (GtkButtonClass   *klass);
-static void gtk_button_init           (GtkButton        *button);
-static void gtk_button_destroy        (GtkObject        *object);
-static void gtk_button_set_property   (GObject         *object,
-                                       guint            prop_id,
-                                       const GValue    *value,
-                                       GParamSpec      *pspec);
-static void gtk_button_get_property   (GObject         *object,
-                                       guint            prop_id,
-                                       GValue          *value,
-                                       GParamSpec      *pspec);
-static void gtk_button_screen_changed (GtkWidget        *widget,
-				       GdkScreen        *previous_screen);
-static void gtk_button_realize        (GtkWidget        *widget);
-static void gtk_button_unrealize      (GtkWidget        *widget);
-static void gtk_button_map            (GtkWidget        *widget);
-static void gtk_button_unmap          (GtkWidget        *widget);
-static void gtk_button_size_request   (GtkWidget        *widget,
-				       GtkRequisition   *requisition);
-static void gtk_button_size_allocate  (GtkWidget        *widget,
-				       GtkAllocation    *allocation);
-static gint gtk_button_expose         (GtkWidget        *widget,
-				       GdkEventExpose   *event);
-static gint gtk_button_button_press   (GtkWidget        *widget,
-				       GdkEventButton   *event);
-static gint gtk_button_button_release (GtkWidget        *widget,
-				       GdkEventButton   *event);
-static gint gtk_button_key_release    (GtkWidget        *widget,
-				       GdkEventKey      *event);
-static gint gtk_button_enter_notify   (GtkWidget        *widget,
-				       GdkEventCrossing *event);
-static gint gtk_button_leave_notify   (GtkWidget        *widget,
-				       GdkEventCrossing *event);
-static void gtk_real_button_pressed   (GtkButton        *button);
-static void gtk_real_button_released  (GtkButton        *button);
-static void gtk_real_button_activate  (GtkButton         *button);
-static void gtk_button_update_state   (GtkButton        *button);
-static void gtk_button_add            (GtkContainer   *container,
-			               GtkWidget      *widget);
-static GType gtk_button_child_type    (GtkContainer     *container);
-static void gtk_button_finish_activate (GtkButton *button,
-					gboolean   do_it);
+static void gtk_button_destroy        (GtkObject          *object);
+static void gtk_button_set_property   (GObject            *object,
+                                       guint               prop_id,
+                                       const GValue       *value,
+                                       GParamSpec         *pspec);
+static void gtk_button_get_property   (GObject            *object,
+                                       guint               prop_id,
+                                       GValue             *value,
+                                       GParamSpec         *pspec);
+static void gtk_button_screen_changed (GtkWidget          *widget,
+				       GdkScreen          *previous_screen);
+static void gtk_button_realize        (GtkWidget          *widget);
+static void gtk_button_unrealize      (GtkWidget          *widget);
+static void gtk_button_map            (GtkWidget          *widget);
+static void gtk_button_unmap          (GtkWidget          *widget);
+static void gtk_button_size_request   (GtkWidget          *widget,
+				       GtkRequisition     *requisition);
+static void gtk_button_size_allocate  (GtkWidget          *widget,
+				       GtkAllocation      *allocation);
+static gint gtk_button_expose         (GtkWidget          *widget,
+				       GdkEventExpose     *event);
+static gint gtk_button_button_press   (GtkWidget          *widget,
+				       GdkEventButton     *event);
+static gint gtk_button_button_release (GtkWidget          *widget,
+				       GdkEventButton     *event);
+static gint gtk_button_grab_broken    (GtkWidget          *widget,
+				       GdkEventGrabBroken *event);
+static gint gtk_button_key_release    (GtkWidget          *widget,
+				       GdkEventKey        *event);
+static gint gtk_button_enter_notify   (GtkWidget          *widget,
+				       GdkEventCrossing   *event);
+static gint gtk_button_leave_notify   (GtkWidget          *widget,
+				       GdkEventCrossing   *event);
+static void gtk_real_button_pressed   (GtkButton          *button);
+static void gtk_real_button_released  (GtkButton          *button);
+static void gtk_real_button_activate  (GtkButton          *button);
+static void gtk_button_update_state   (GtkButton          *button);
+static void gtk_button_add            (GtkContainer       *container,
+			               GtkWidget          *widget);
+static GType gtk_button_child_type    (GtkContainer       *container);
+static void gtk_button_finish_activate (GtkButton         *button,
+					gboolean           do_it);
 
 static GObject*	gtk_button_constructor (GType                  type,
 					guint                  n_construct_properties,
@@ -135,37 +137,9 @@ static void gtk_button_grab_notify     (GtkWidget             *widget,
 					gboolean               was_grabbed);
 
 
-
-static GtkBinClass *parent_class = NULL;
 static guint button_signals[LAST_SIGNAL] = { 0 };
 
-
-GType
-gtk_button_get_type (void)
-{
-  static GType button_type = 0;
-
-  if (!button_type)
-    {
-      static const GTypeInfo button_info =
-      {
-	sizeof (GtkButtonClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_button_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkButton),
-	16,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_button_init,
-      };
-
-      button_type = g_type_register_static (GTK_TYPE_BIN, "GtkButton",
-					    &button_info, 0);
-    }
-
-  return button_type;
-}
+G_DEFINE_TYPE (GtkButton, gtk_button, GTK_TYPE_BIN);
 
 static void
 gtk_button_class_init (GtkButtonClass *klass)
@@ -180,8 +154,6 @@ gtk_button_class_init (GtkButtonClass *klass)
   widget_class = (GtkWidgetClass*) klass;
   container_class = (GtkContainerClass*) klass;
   
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->constructor = gtk_button_constructor;
   gobject_class->set_property = gtk_button_set_property;
   gobject_class->get_property = gtk_button_get_property;
@@ -198,6 +170,7 @@ gtk_button_class_init (GtkButtonClass *klass)
   widget_class->expose_event = gtk_button_expose;
   widget_class->button_press_event = gtk_button_button_press;
   widget_class->button_release_event = gtk_button_button_release;
+  widget_class->grab_broken_event = gtk_button_grab_broken;
   widget_class->key_release_event = gtk_button_key_release;
   widget_class->enter_notify_event = gtk_button_enter_notify;
   widget_class->leave_notify_event = gtk_button_leave_notify;
@@ -220,31 +193,31 @@ gtk_button_class_init (GtkButtonClass *klass)
                                                         P_("Label"),
                                                         P_("Text of the label widget inside the button, if the button contains a label widget"),
                                                         NULL,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                        GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   
   g_object_class_install_property (gobject_class,
                                    PROP_USE_UNDERLINE,
-                                   g_param_spec_boolean ("use_underline",
+                                   g_param_spec_boolean ("use-underline",
 							 P_("Use underline"),
 							 P_("If set, an underline in the text indicates the next character should be used for the mnemonic accelerator key"),
                                                         FALSE,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                        GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   
   g_object_class_install_property (gobject_class,
                                    PROP_USE_STOCK,
-                                   g_param_spec_boolean ("use_stock",
+                                   g_param_spec_boolean ("use-stock",
 							 P_("Use stock"),
 							 P_("If set, the label is used to pick a stock item instead of being displayed"),
                                                         FALSE,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                        GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   
   g_object_class_install_property (gobject_class,
                                    PROP_FOCUS_ON_CLICK,
-                                   g_param_spec_boolean ("focus_on_click",
+                                   g_param_spec_boolean ("focus-on-click",
 							 P_("Focus on click"),
 							 P_("Whether the button grabs focus when it is clicked with the mouse"),
 							 TRUE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_RELIEF,
@@ -253,7 +226,7 @@ gtk_button_class_init (GtkButtonClass *klass)
                                                       P_("The border relief style"),
                                                       GTK_TYPE_RELIEF_STYLE,
                                                       GTK_RELIEF_NORMAL,
-                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                      GTK_PARAM_READWRITE));
   
   /**
    * GtkButton:xalign:
@@ -272,7 +245,7 @@ gtk_button_class_init (GtkButtonClass *klass)
                                                       0.0,
                                                       1.0,
                                                       0.5,
-                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                      GTK_PARAM_READWRITE));
 
   /**
    * GtkButton:yalign:
@@ -291,7 +264,7 @@ gtk_button_class_init (GtkButtonClass *klass)
                                                       0.0,
                                                       1.0,
                                                       0.5,
-                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                      GTK_PARAM_READWRITE));
 
   /**
    * GtkButton::image:
@@ -306,42 +279,84 @@ gtk_button_class_init (GtkButtonClass *klass)
                                                         P_("Image widget"),
                                                         P_("Child widget to appear next to the button text"),
                                                         GTK_TYPE_WIDGET,
-                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                        GTK_PARAM_READWRITE));
 
+  /**
+   * GtkButton::pressed:
+   * @button: the object that received the signal
+   *
+   * Emitted when the button is pressed.
+   * 
+   * @Deprecated: Use the GtkWidget::button-press-event signal.
+   */ 
   button_signals[PRESSED] =
-    g_signal_new ("pressed",
+    g_signal_new (I_("pressed"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkButtonClass, pressed),
 		  NULL, NULL,
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
+
+  /**
+   * GtkButton::released:
+   * @button: the object that received the signal
+   *
+   * Emitted when the button is released.
+   * 
+   * @Deprecated: Use the GtkWidget::button-release-event signal.
+   */ 
   button_signals[RELEASED] =
-    g_signal_new ("released",
+    g_signal_new (I_("released"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkButtonClass, released),
 		  NULL, NULL,
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
+
+  /**
+   * GtkButton::clicked:
+   * @button: the object that received the signal
+   *
+   * Emitted when the button has been activated (pressed and released).
+   */ 
   button_signals[CLICKED] =
-    g_signal_new ("clicked",
+    g_signal_new (I_("clicked"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GtkButtonClass, clicked),
 		  NULL, NULL,
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
+
+  /**
+   * GtkButton::enter:
+   * @button: the object that received the signal
+   *
+   * Emitted when the pointer enters the button.
+   * 
+   * @Deprecated: Use the GtkWidget::enter-notify-event signal.
+   */ 
   button_signals[ENTER] =
-    g_signal_new ("enter",
+    g_signal_new (I_("enter"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkButtonClass, enter),
 		  NULL, NULL,
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
+
+  /**
+   * GtkButton::leave:
+   * @button: the object that received the signal
+   *
+   * Emitted when the pointer leaves the button.
+   * 
+   * @Deprecated: Use the GtkWidget::leave-notify-event signal.
+   */ 
   button_signals[LEAVE] =
-    g_signal_new ("leave",
+    g_signal_new (I_("leave"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkButtonClass, leave),
@@ -359,7 +374,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    * "clicked" signal.
    */
   button_signals[ACTIVATE] =
-    g_signal_new ("activate",
+    g_signal_new (I_("activate"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GtkButtonClass, activate),
@@ -369,34 +384,34 @@ gtk_button_class_init (GtkButtonClass *klass)
   widget_class->activate_signal = button_signals[ACTIVATE];
 
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_boxed ("default_border",
+					   g_param_spec_boxed ("default-border",
 							       P_("Default Spacing"),
 							       P_("Extra space to add for CAN_DEFAULT buttons"),
 							       GTK_TYPE_BORDER,
-							       G_PARAM_READABLE));
+							       GTK_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_boxed ("default_outside_border",
+					   g_param_spec_boxed ("default-outside-border",
 							       P_("Default Outside Spacing"),
 							       P_("Extra space to add for CAN_DEFAULT buttons that is always drawn outside the border"),
 							       GTK_TYPE_BORDER,
-							       G_PARAM_READABLE));
+							       GTK_PARAM_READABLE));
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("child_displacement_x",
+					   g_param_spec_int ("child-displacement-x",
 							     P_("Child X Displacement"),
 							     P_("How far in the x direction to move the child when the button is depressed"),
 							     G_MININT,
 							     G_MAXINT,
 							     0,
-							     G_PARAM_READABLE));
+							     GTK_PARAM_READABLE));
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("child_displacement_y",
+					   g_param_spec_int ("child-displacement-y",
 							     P_("Child Y Displacement"),
 							     P_("How far in the y direction to move the child when the button is depressed"),
 							     G_MININT,
 							     G_MAXINT,
 							     0,
-							     G_PARAM_READABLE));
+							     GTK_PARAM_READABLE));
 
   /**
    * GtkButton:displace-focus:
@@ -411,15 +426,29 @@ gtk_button_class_init (GtkButtonClass *klass)
 								 P_("Displace focus"),
 								 P_("Whether the child_displacement_x/_y properties should also affect the focus rectangle"),
 						       FALSE,
-						       G_PARAM_READABLE));
+						       GTK_PARAM_READABLE));
+
+  /**
+   * GtkButton:inner-border:
+   *
+   * Sets the border between the button edges and child.
+   *
+   * Since: 2.10
+   */
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_boxed ("inner-border",
+                                                               P_("Inner Border"),
+                                                               P_("Border between button edges and child."),
+                                                               GTK_TYPE_BORDER,
+                                                               GTK_PARAM_READABLE));
 
   gtk_settings_install_property (g_param_spec_boolean ("gtk-button-images",
 						       P_("Show button images"),
 						       P_("Whether stock icons should be shown in buttons"),
 						       TRUE,
-						       G_PARAM_READWRITE));
-  
-  g_type_class_add_private (gobject_class, sizeof (GtkButtonPrivate));  
+						       GTK_PARAM_READWRITE));
+
+  g_type_class_add_private (gobject_class, sizeof (GtkButtonPrivate));
 }
 
 static void
@@ -459,7 +488,7 @@ gtk_button_destroy (GtkObject *object)
       button->label_text = NULL;
     }
   
-  (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+  (* GTK_OBJECT_CLASS (gtk_button_parent_class)->destroy) (object);
 }
 
 static GObject*
@@ -470,9 +499,9 @@ gtk_button_constructor (GType                  type,
   GObject *object;
   GtkButton *button;
 
-  object = (* G_OBJECT_CLASS (parent_class)->constructor) (type,
-							   n_construct_properties,
-							   construct_params);
+  object = (* G_OBJECT_CLASS (gtk_button_parent_class)->constructor) (type,
+								      n_construct_properties,
+								      construct_params);
 
   button = GTK_BUTTON (object);
   button->constructed = TRUE;
@@ -522,7 +551,7 @@ gtk_button_add (GtkContainer *container,
 {
   maybe_set_alignment (GTK_BUTTON (container), widget);
 
-  GTK_CONTAINER_CLASS (parent_class)->add (container, widget);
+  GTK_CONTAINER_CLASS (gtk_button_parent_class)->add (container, widget);
 }
 
 static void
@@ -616,10 +645,17 @@ gtk_button_new (void)
 static gboolean
 show_image (GtkButton *button)
 {
-  GtkSettings *settings = gtk_widget_get_settings (GTK_WIDGET (button));  
   gboolean show;
+  
+  if (button->label_text)
+    {
+      GtkSettings *settings;
 
-  g_object_get (settings, "gtk-button-images", &show, NULL);
+      settings = gtk_widget_get_settings (GTK_WIDGET (button));        
+      g_object_get (settings, "gtk-button-images", &show, NULL);
+    }
+  else
+    show = TRUE;
 
   return show;
 }
@@ -637,22 +673,25 @@ gtk_button_construct_child (GtkButton *button)
   
   if (!button->constructed)
     return;
-  
-  if (button->label_text == NULL)
+ 
+  if (!button->label_text && !priv->image)
     return;
-
-  if (GTK_BIN (button)->child)
-    {
-      if (priv->image && !priv->image_is_stock)
-	image = g_object_ref (priv->image);
-
-      gtk_container_remove (GTK_CONTAINER (button),
-  			    GTK_BIN (button)->child);
   
+  if (priv->image && !priv->image_is_stock)
+    {
+      image = g_object_ref (priv->image);
+      if (image->parent)
+	gtk_container_remove (GTK_CONTAINER (image->parent), image);
+      
       priv->image = NULL;
     }
   
+  if (GTK_BIN (button)->child)
+    gtk_container_remove (GTK_CONTAINER (button),
+			  GTK_BIN (button)->child);
+  
   if (button->use_stock &&
+      button->label_text &&
       gtk_stock_lookup (button->label_text, &item))
     {
       if (!image)
@@ -665,14 +704,11 @@ gtk_button_construct_child (GtkButton *button)
 
   if (image)
     {
-      label = gtk_label_new_with_mnemonic (label_text);
-      gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
-      
       priv->image = image;
 
       g_object_set (priv->image, 
 		    "visible", show_image (button),
-		    "no_show_all", TRUE,
+		    "no-show-all", TRUE,
 		    NULL);
       hbox = gtk_hbox_new (FALSE, 2);
 
@@ -682,7 +718,15 @@ gtk_button_construct_child (GtkButton *button)
 	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 	
       gtk_box_pack_start (GTK_BOX (hbox), priv->image, FALSE, FALSE, 0);
-      gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+      if (label_text)
+	{
+	  label = gtk_label_new_with_mnemonic (label_text);
+	  gtk_label_set_mnemonic_widget (GTK_LABEL (label), 
+					 GTK_WIDGET (button));
+
+	  gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	}
       
       gtk_container_add (GTK_CONTAINER (button), align);
       gtk_container_add (GTK_CONTAINER (align), hbox);
@@ -693,7 +737,7 @@ gtk_button_construct_child (GtkButton *button)
       return;
     }
   
- if (button->use_underline)
+  if (button->use_underline)
     {
       label = gtk_label_new_with_mnemonic (button->label_text);
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
@@ -703,7 +747,7 @@ gtk_button_construct_child (GtkButton *button)
   
   if (priv->align_set)
     gtk_misc_set_alignment (GTK_MISC (label), priv->xalign, priv->yalign);
-
+  
   gtk_widget_show (label);
   gtk_container_add (GTK_CONTAINER (button), label);
 }
@@ -733,8 +777,8 @@ gtk_button_new_from_stock (const gchar *stock_id)
 {
   return g_object_new (GTK_TYPE_BUTTON,
                        "label", stock_id,
-                       "use_stock", TRUE,
-                       "use_underline", TRUE,
+                       "use-stock", TRUE,
+                       "use-underline", TRUE,
                        NULL);
 }
 
@@ -754,7 +798,7 @@ gtk_button_new_from_stock (const gchar *stock_id)
 GtkWidget*
 gtk_button_new_with_mnemonic (const gchar *label)
 {
-  return g_object_new (GTK_TYPE_BUTTON, "label", label, "use_underline", TRUE,  NULL);
+  return g_object_new (GTK_TYPE_BUTTON, "label", label, "use-underline", TRUE,  NULL);
 }
 
 void
@@ -872,7 +916,7 @@ gtk_button_unrealize (GtkWidget *widget)
       button->event_window = NULL;
     }
   
-  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+  GTK_WIDGET_CLASS (gtk_button_parent_class)->unrealize (widget);
 }
 
 static void
@@ -880,7 +924,7 @@ gtk_button_map (GtkWidget *widget)
 {
   GtkButton *button = GTK_BUTTON (widget);
   
-  GTK_WIDGET_CLASS (parent_class)->map (widget);
+  GTK_WIDGET_CLASS (gtk_button_parent_class)->map (widget);
 
   if (button->event_window)
     gdk_window_show (button->event_window);
@@ -894,13 +938,14 @@ gtk_button_unmap (GtkWidget *widget)
   if (button->event_window)
     gdk_window_hide (button->event_window);
 
-  GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+  GTK_WIDGET_CLASS (gtk_button_parent_class)->unmap (widget);
 }
 
 static void
 gtk_button_get_props (GtkButton *button,
 		      GtkBorder *default_border,
 		      GtkBorder *default_outside_border,
+                      GtkBorder *inner_border,
 		      gboolean  *interior_focus)
 {
   GtkWidget *widget =  GTK_WIDGET (button);
@@ -908,7 +953,7 @@ gtk_button_get_props (GtkButton *button,
 
   if (default_border)
     {
-      gtk_widget_style_get (widget, "default_border", &tmp_border, NULL);
+      gtk_widget_style_get (widget, "default-border", &tmp_border, NULL);
 
       if (tmp_border)
 	{
@@ -921,7 +966,7 @@ gtk_button_get_props (GtkButton *button,
 
   if (default_outside_border)
     {
-      gtk_widget_style_get (widget, "default_outside_border", &tmp_border, NULL);
+      gtk_widget_style_get (widget, "default-outside-border", &tmp_border, NULL);
 
       if (tmp_border)
 	{
@@ -932,8 +977,21 @@ gtk_button_get_props (GtkButton *button,
 	*default_outside_border = default_default_outside_border;
     }
 
+  if (inner_border)
+    {
+      gtk_widget_style_get (widget, "inner-border", &tmp_border, NULL);
+
+      if (tmp_border)
+	{
+	  *inner_border = *tmp_border;
+	  g_free (tmp_border);
+	}
+      else
+	*inner_border = default_inner_border;
+    }
+
   if (interior_focus)
-    gtk_widget_style_get (widget, "interior_focus", interior_focus, NULL);
+    gtk_widget_style_get (widget, "interior-focus", interior_focus, NULL);
 }
 	
 static void
@@ -942,19 +1000,22 @@ gtk_button_size_request (GtkWidget      *widget,
 {
   GtkButton *button = GTK_BUTTON (widget);
   GtkBorder default_border;
+  GtkBorder inner_border;
   gint focus_width;
   gint focus_pad;
 
-  gtk_button_get_props (button, &default_border, NULL, NULL);
+  gtk_button_get_props (button, &default_border, NULL, &inner_border, NULL);
   gtk_widget_style_get (GTK_WIDGET (widget),
 			"focus-line-width", &focus_width,
 			"focus-padding", &focus_pad,
 			NULL);
  
-  requisition->width = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
-			GTK_WIDGET (widget)->style->xthickness) * 2;
-  requisition->height = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
-			 GTK_WIDGET (widget)->style->ythickness) * 2;
+  requisition->width = ((GTK_CONTAINER (widget)->border_width +
+                         GTK_WIDGET (widget)->style->xthickness) * 2 +
+                        inner_border.left + inner_border.right);
+  requisition->height = ((GTK_CONTAINER (widget)->border_width +
+                          GTK_WIDGET (widget)->style->ythickness) * 2 +
+                         inner_border.top + inner_border.bottom);
 
   if (GTK_WIDGET_CAN_DEFAULT (widget))
     {
@@ -987,10 +1048,11 @@ gtk_button_size_allocate (GtkWidget     *widget,
   gint xthickness = GTK_WIDGET (widget)->style->xthickness;
   gint ythickness = GTK_WIDGET (widget)->style->ythickness;
   GtkBorder default_border;
+  GtkBorder inner_border;
   gint focus_width;
   gint focus_pad;
 
-  gtk_button_get_props (button, &default_border, NULL, NULL);
+  gtk_button_get_props (button, &default_border, NULL, &inner_border, NULL);
   gtk_widget_style_get (GTK_WIDGET (widget),
 			"focus-line-width", &focus_width,
 			"focus-padding", &focus_pad,
@@ -1008,12 +1070,18 @@ gtk_button_size_allocate (GtkWidget     *widget,
 
   if (GTK_BIN (button)->child && GTK_WIDGET_VISIBLE (GTK_BIN (button)->child))
     {
-      child_allocation.x = widget->allocation.x + border_width + CHILD_SPACING + xthickness;
-      child_allocation.y = widget->allocation.y + border_width + CHILD_SPACING + ythickness;
+      child_allocation.x = widget->allocation.x + border_width + inner_border.left + xthickness;
+      child_allocation.y = widget->allocation.y + border_width + inner_border.top + ythickness;
       
-      child_allocation.width = MAX (1, widget->allocation.width - (CHILD_SPACING + xthickness) * 2 -
+      child_allocation.width = MAX (1, widget->allocation.width -
+                                    xthickness * 2 -
+                                    inner_border.left -
+                                    inner_border.right -
 				    border_width * 2);
-      child_allocation.height = MAX (1, widget->allocation.height - (CHILD_SPACING + ythickness) * 2 -
+      child_allocation.height = MAX (1, widget->allocation.height -
+                                     ythickness * 2 -
+                                     inner_border.top -
+                                     inner_border.bottom -
 				     border_width * 2);
 
       if (GTK_WIDGET_CAN_DEFAULT (button))
@@ -1038,8 +1106,8 @@ gtk_button_size_allocate (GtkWidget     *widget,
 	  gint child_displacement_y;
 	  
 	  gtk_widget_style_get (widget,
-				"child_displacement_x", &child_displacement_x, 
-				"child_displacement_y", &child_displacement_y,
+				"child-displacement-x", &child_displacement_x, 
+				"child-displacement-y", &child_displacement_y,
 				NULL);
 	  child_allocation.x += child_displacement_x;
 	  child_allocation.y += child_displacement_y;
@@ -1072,7 +1140,7 @@ _gtk_button_paint (GtkButton    *button,
       widget = GTK_WIDGET (button);
       border_width = GTK_CONTAINER (widget)->border_width;
 
-      gtk_button_get_props (button, &default_border, &default_outside_border, &interior_focus);
+      gtk_button_get_props (button, &default_border, &default_outside_border, NULL, &interior_focus);
       gtk_widget_style_get (GTK_WIDGET (widget),
 			    "focus-line-width", &focus_width,
 			    "focus-padding", &focus_pad,
@@ -1126,9 +1194,9 @@ _gtk_button_paint (GtkButton    *button,
 	  gboolean displace_focus;
 	  
 	  gtk_widget_style_get (GTK_WIDGET (widget),
-				"child_displacement_y", &child_displacement_y,
-				"child_displacement_x", &child_displacement_x,
-				"displace_focus", &displace_focus,
+				"child-displacement-y", &child_displacement_y,
+				"child-displacement-x", &child_displacement_x,
+				"displace-focus", &displace_focus,
 				NULL);
 
 	  if (interior_focus)
@@ -1172,7 +1240,7 @@ gtk_button_expose (GtkWidget      *widget,
 			 button->depressed ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
 			 "button", "buttondefault");
       
-      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
+      (* GTK_WIDGET_CLASS (gtk_button_parent_class)->expose_event) (widget, event);
     }
   
   return FALSE;
@@ -1213,6 +1281,29 @@ gtk_button_button_release (GtkWidget      *widget,
   return TRUE;
 }
 
+static gboolean
+gtk_button_grab_broken (GtkWidget          *widget,
+			GdkEventGrabBroken *event)
+{
+  GtkButton *button = GTK_BUTTON (widget);
+  gboolean save_in;
+  
+  /* Simulate a button release without the pointer in the button */
+  if (button->button_down)
+    {
+      save_in = button->in_button;
+      button->in_button = FALSE;
+      gtk_button_released (button);
+      if (save_in != button->in_button)
+	{
+	  button->in_button = save_in;
+	  gtk_button_update_state (button);
+	}
+    }
+
+  return TRUE;
+}
+
 static gboolean
 gtk_button_key_release (GtkWidget   *widget,
 			GdkEventKey *event)
@@ -1224,8 +1315,8 @@ gtk_button_key_release (GtkWidget   *widget,
       gtk_button_finish_activate (button, TRUE);
       return TRUE;
     }
-  else if (GTK_WIDGET_CLASS (parent_class)->key_release_event)
-    return GTK_WIDGET_CLASS (parent_class)->key_release_event (widget, event);
+  else if (GTK_WIDGET_CLASS (gtk_button_parent_class)->key_release_event)
+    return GTK_WIDGET_CLASS (gtk_button_parent_class)->key_release_event (widget, event);
   else
     return FALSE;
 }
@@ -1313,21 +1404,29 @@ static void
 gtk_real_button_activate (GtkButton *button)
 {
   GtkWidget *widget = GTK_WIDGET (button);
-  
+  GtkButtonPrivate *priv;
+  guint32 time;
+
+  priv = GTK_BUTTON_GET_PRIVATE (button);
+
   if (GTK_WIDGET_REALIZED (button) && !button->activate_timeout)
     {
-      if (gdk_keyboard_grab (button->event_window, TRUE,
-			     gtk_get_current_event_time ()) == 0)
+      time = gtk_get_current_event_time ();
+      if (gdk_keyboard_grab (button->event_window, TRUE, time) == 
+	  GDK_GRAB_SUCCESS)
 	{
-	  gtk_grab_add (widget);
-	  
-	  button->activate_timeout = g_timeout_add (ACTIVATE_TIMEOUT,
-						    button_activate_timeout,
-						    button);
-	  button->button_down = TRUE;
-	  gtk_button_update_state (button);
-	  gtk_widget_queue_draw (GTK_WIDGET (button));
+	  priv->has_grab = TRUE;
+	  priv->grab_time = time;
 	}
+
+      gtk_grab_add (widget);
+      
+      button->activate_timeout = g_timeout_add (ACTIVATE_TIMEOUT,
+						button_activate_timeout,
+						button);
+      button->button_down = TRUE;
+      gtk_button_update_state (button);
+      gtk_widget_queue_draw (GTK_WIDGET (button));
     }
 }
 
@@ -1336,12 +1435,18 @@ gtk_button_finish_activate (GtkButton *button,
 			    gboolean   do_it)
 {
   GtkWidget *widget = GTK_WIDGET (button);
+  GtkButtonPrivate *priv;
   
+  priv = GTK_BUTTON_GET_PRIVATE (button);
+
   g_source_remove (button->activate_timeout);
   button->activate_timeout = 0;
 
-  gdk_display_keyboard_ungrab (gtk_widget_get_display (widget),
-			       gtk_get_current_event_time ());
+  if (priv->has_grab)
+    {
+      gdk_display_keyboard_ungrab (gtk_widget_get_display (widget),
+				   priv->grab_time);
+    }
   gtk_grab_remove (widget);
 
   button->button_down = FALSE;
@@ -1708,7 +1813,7 @@ gtk_button_screen_changed (GtkWidget *widget,
     g_signal_connect (settings, "notify::gtk-button-images",
 		      G_CALLBACK (gtk_button_setting_changed), 0);
   g_object_set_data (G_OBJECT (settings), 
-		     "gtk-button-connection",
+		     I_("gtk-button-connection"),
 		     GUINT_TO_POINTER (show_image_connection));
 
   show_image_change_notify (GTK_BUTTON (widget));
@@ -1747,7 +1852,8 @@ gtk_button_grab_notify (GtkWidget *widget,
  *
  * Set the image of @button to the given widget. Note that
  * it depends on the gtk-button-images setting whether the
- * image will be displayed or not.
+ * image will be displayed or not, you don't have to call
+ * gtk_widget_show() on @image yourself.
  *
  * Since: 2.6
  */ 
@@ -1755,7 +1861,12 @@ void
 gtk_button_set_image (GtkButton *button,
 		      GtkWidget *image)
 {
-  GtkButtonPrivate *priv = GTK_BUTTON_GET_PRIVATE (button);
+  GtkButtonPrivate *priv;
+
+  g_return_if_fail (GTK_IS_BUTTON (button));
+  g_return_if_fail (image == NULL || GTK_IS_WIDGET (image));
+
+  priv = GTK_BUTTON_GET_PRIVATE (button);
 
   priv->image = image;
   priv->image_is_stock = (image == NULL);
