@@ -25,8 +25,6 @@
 #include "gtktreeprivate.h"
 #include "gtkalias.h"
 
-static void gtk_cell_renderer_init       (GtkCellRenderer      *cell);
-static void gtk_cell_renderer_class_init (GtkCellRendererClass *class);
 static void gtk_cell_renderer_get_property  (GObject              *object,
 					     guint                 param_id,
 					     GValue               *value,
@@ -75,34 +73,7 @@ enum {
 
 static guint cell_renderer_signals[LAST_SIGNAL] = { 0 };
 
-
-GType
-gtk_cell_renderer_get_type (void)
-{
-  static GType cell_type = 0;
-
-  if (!cell_type)
-    {
-      static const GTypeInfo cell_info =
-      {
-        sizeof (GtkCellRendererClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-        (GClassInitFunc) gtk_cell_renderer_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-        sizeof (GtkCellRenderer),
-	0,		/* n_preallocs */
-        (GInstanceInitFunc) gtk_cell_renderer_init,
-	NULL,		/* value_table */
-      };
-
-      cell_type = g_type_register_static (GTK_TYPE_OBJECT, "GtkCellRenderer", 
-					  &cell_info, G_TYPE_FLAG_ABSTRACT);
-    }
-
-  return cell_type;
-}
+G_DEFINE_ABSTRACT_TYPE (GtkCellRenderer, gtk_cell_renderer, GTK_TYPE_OBJECT);
 
 static void
 gtk_cell_renderer_init (GtkCellRenderer *cell)
@@ -145,7 +116,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
    * Since: 2.4
    */
   cell_renderer_signals[EDITING_CANCELED] =
-    g_signal_new ("editing-canceled",
+    g_signal_new (I_("editing-canceled"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkCellRendererClass, editing_canceled),
@@ -190,7 +161,7 @@ gtk_cell_renderer_class_init (GtkCellRendererClass *class)
    * Since: 2.6
    */
   cell_renderer_signals[EDITING_STARTED] =
-    g_signal_new ("editing-started",
+    g_signal_new (I_("editing-started"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkCellRendererClass, editing_started),
@@ -758,7 +729,7 @@ gtk_cell_renderer_get_fixed_size (GtkCellRenderer *cell,
  * changes were not committed.
  *
  * Since: 2.4
- * Deprecated: Use gtk_cell_renderer_stop_editing() instead
+ * Deprecated: 2.6: Use gtk_cell_renderer_stop_editing() instead
  **/
 void
 gtk_cell_renderer_editing_canceled (GtkCellRenderer *cell)
