@@ -24,6 +24,7 @@
 #include "gtkentry.h"
 #include "gtkcellrenderertext.h"
 
+#include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkalias.h"
 
@@ -37,9 +38,6 @@ struct _GtkComboBoxEntryPrivate
   gint text_column;
 };
 
-static void gtk_combo_box_entry_class_init       (GtkComboBoxEntryClass *klass);
-static void gtk_combo_box_entry_init             (GtkComboBoxEntry      *entry_box);
-
 static void gtk_combo_box_entry_set_property     (GObject               *object,
                                                   guint                  prop_id,
                                                   const GValue          *value,
@@ -67,35 +65,7 @@ enum
   PROP_TEXT_COLUMN
 };
 
-
-GType
-gtk_combo_box_entry_get_type (void)
-{
-  static GType combo_box_entry_type = 0;
-
-  if (!combo_box_entry_type)
-    {
-      static const GTypeInfo combo_box_entry_info =
-        {
-          sizeof (GtkComboBoxEntryClass),
-          NULL, /* base_init */
-          NULL, /* base_finalize */
-          (GClassInitFunc) gtk_combo_box_entry_class_init,
-          NULL, /* class_finalize */
-          NULL, /* class_data */
-          sizeof (GtkComboBoxEntry),
-          0,
-          (GInstanceInitFunc) gtk_combo_box_entry_init
-        };
-
-      combo_box_entry_type = g_type_register_static (GTK_TYPE_COMBO_BOX,
-                                                     "GtkComboBoxEntry",
-                                                     &combo_box_entry_info,
-                                                     0);
-    }
-
-  return combo_box_entry_type;
-}
+G_DEFINE_TYPE (GtkComboBoxEntry, gtk_combo_box_entry, GTK_TYPE_COMBO_BOX);
 
 static void
 gtk_combo_box_entry_class_init (GtkComboBoxEntryClass *klass)
@@ -103,7 +73,7 @@ gtk_combo_box_entry_class_init (GtkComboBoxEntryClass *klass)
   GObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkComboBoxClass *combo_class;
-
+  
   object_class = (GObjectClass *)klass;
   object_class->set_property = gtk_combo_box_entry_set_property;
   object_class->get_property = gtk_combo_box_entry_get_property;
@@ -117,13 +87,13 @@ gtk_combo_box_entry_class_init (GtkComboBoxEntryClass *klass)
   
   g_object_class_install_property (object_class,
                                    PROP_TEXT_COLUMN,
-                                   g_param_spec_int ("text_column",
+                                   g_param_spec_int ("text-column",
                                                      P_("Text Column"),
                                                      P_("A column in the data source model to get the strings from"),
                                                      -1,
                                                      G_MAXINT,
                                                      -1,
-                                                     G_PARAM_READWRITE));
+                                                     GTK_PARAM_READWRITE));
 
   g_type_class_add_private ((GObjectClass *) klass,
                             sizeof (GtkComboBoxEntryPrivate));
@@ -233,7 +203,7 @@ has_frame_changed (GtkComboBoxEntry *entry_box,
 {
   gboolean has_frame;
   
-  g_object_get (entry_box, "has_frame", &has_frame, NULL);
+  g_object_get (entry_box, "has-frame", &has_frame, NULL);
 
   gtk_entry_set_has_frame (GTK_ENTRY (entry_box->priv->entry), has_frame);
 }
@@ -299,7 +269,7 @@ gtk_combo_box_entry_new_with_model (GtkTreeModel *model,
 
   ret = g_object_new (gtk_combo_box_entry_get_type (),
                       "model", model,
-                      "text_column", text_column,
+                      "text-column", text_column,
                       NULL);
 
   return ret;
