@@ -52,9 +52,6 @@ struct _ResponseData
   gint response_id;
 };
 
-static void gtk_dialog_class_init (GtkDialogClass *klass);
-static void gtk_dialog_init       (GtkDialog      *dialog);
-
 static void gtk_dialog_add_buttons_valist (GtkDialog   *dialog,
                                            const gchar *first_button_text,
                                            va_list      args);
@@ -91,35 +88,9 @@ enum {
   LAST_SIGNAL
 };
 
-static gpointer parent_class;
 static guint dialog_signals[LAST_SIGNAL];
 
-GType
-gtk_dialog_get_type (void)
-{
-  static GType dialog_type = 0;
-
-  if (!dialog_type)
-    {
-      static const GTypeInfo dialog_info =
-      {
-	sizeof (GtkDialogClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_dialog_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkDialog),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_dialog_init,
-      };
-
-      dialog_type = g_type_register_static (GTK_TYPE_WINDOW, "GtkDialog",
-					    &dialog_info, 0);
-    }
-
-  return dialog_type;
-}
+G_DEFINE_TYPE (GtkDialog, gtk_dialog, GTK_TYPE_WINDOW);
 
 static void
 gtk_dialog_class_init (GtkDialogClass *class)
@@ -131,8 +102,6 @@ gtk_dialog_class_init (GtkDialogClass *class)
   gobject_class = G_OBJECT_CLASS (class);
   widget_class = GTK_WIDGET_CLASS (class);
   
-  parent_class = g_type_class_peek_parent (class);
-
   gobject_class->set_property = gtk_dialog_set_property;
   gobject_class->get_property = gtk_dialog_get_property;
   
@@ -152,7 +121,7 @@ gtk_dialog_class_init (GtkDialogClass *class)
                                                          GTK_PARAM_READWRITE));
   
   dialog_signals[RESPONSE] =
-    g_signal_new ("response",
+    g_signal_new (I_("response"),
 		  G_OBJECT_CLASS_TYPE (class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkDialogClass, response),
@@ -162,7 +131,7 @@ gtk_dialog_class_init (GtkDialogClass *class)
 		  G_TYPE_INT);
 
   dialog_signals[CLOSE] =
-    g_signal_new ("close",
+    g_signal_new (I_("close"),
 		  G_OBJECT_CLASS_TYPE (class),
 		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GtkDialogClass, close),
@@ -338,7 +307,7 @@ gtk_dialog_map (GtkWidget *widget)
   GtkWindow *window = GTK_WINDOW (widget);
   GtkDialog *dialog = GTK_DIALOG (widget);
   
-  GTK_WIDGET_CLASS (parent_class)->map (widget);
+  GTK_WIDGET_CLASS (gtk_dialog_parent_class)->map (widget);
 
   if (!window->focus_widget)
     {
@@ -353,8 +322,13 @@ gtk_dialog_map (GtkWidget *widget)
 	    first_focus = window->focus_widget;
 	  else if (first_focus == window->focus_widget)
 	    break;
+
+	  if (!GTK_IS_LABEL (window->focus_widget))
+	    break;
+	  else
+	    gtk_label_select_region (GTK_LABEL (window->focus_widget), 0, 0);
 	}
-      while (GTK_IS_LABEL (window->focus_widget));
+      while (TRUE);
 
       tmp_list = children = gtk_container_get_children (GTK_CONTAINER (dialog->action_area));
       
@@ -418,9 +392,6 @@ gtk_dialog_close (GtkDialog *dialog)
   GtkWidget *widget = GTK_WIDGET (dialog);
   GdkEvent *event;
 
-  if (!dialog_find_button (dialog, GTK_RESPONSE_CANCEL))
-    return;
-
   event = gdk_event_new (GDK_DELETE);
   
   event->any.window = g_object_ref (widget->window);
@@ -527,6 +498,12 @@ gtk_dialog_new_with_buttons (const gchar    *title,
   return GTK_WIDGET (dialog);
 }
 
+static void 
+response_data_free (gpointer data)
+{
+  g_slice_free (ResponseData, data);
+}
+
 static ResponseData*
 get_response_data (GtkWidget *widget,
 		   gboolean   create)
@@ -536,12 +513,12 @@ get_response_data (GtkWidget *widget,
 
   if (ad == NULL && create)
     {
-      ad = g_new (ResponseData, 1);
+      ad = g_slice_new (ResponseData);
       
       g_object_set_data_full (G_OBJECT (widget),
-                              "gtk-dialog-response-data",
+                              I_("gtk-dialog-response-data"),
                               ad,
-                              g_free);
+			      response_data_free);
     }
 
   return ad;
@@ -962,9 +939,9 @@ run_destroy_handler (GtkDialog *dialog, gpointer data)
  * 
  * Note that even though the recursive main loop gives the effect of a
  * modal dialog (it prevents the user from interacting with other 
- * windows while the dialog is run), callbacks such as timeouts, 
- * IO channel watches, DND drops, etc, <emphasis>will</emphasis> be 
- * triggered during a gtk_dialog_run() call.
+ * windows in the same window group while the dialog is run), callbacks 
+ * such as timeouts, IO channel watches, DND drops, etc, <emphasis>will</emphasis> 
+ * be triggered during a gtk_dialog_run() call.
  * 
  * Return value: response ID
  **/
