@@ -65,9 +65,7 @@ enum
 
 #define GTK_ENTRY_COMPLETION_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_ENTRY_COMPLETION, GtkEntryCompletionPrivate))
 
-static void     gtk_entry_completion_class_init          (GtkEntryCompletionClass *klass);
 static void     gtk_entry_completion_cell_layout_init    (GtkCellLayoutIface      *iface);
-static void     gtk_entry_completion_init                (GtkEntryCompletion      *completion);
 static void     gtk_entry_completion_set_property        (GObject                 *object,
                                                           guint                    prop_id,
                                                           const GValue            *value,
@@ -139,55 +137,17 @@ static gboolean gtk_entry_completion_match_selected      (GtkEntryCompletion *co
 static gboolean gtk_entry_completion_real_insert_prefix  (GtkEntryCompletion *completion,
 							  const gchar        *prefix);
 
-static GObjectClass *parent_class = NULL;
 static guint entry_completion_signals[LAST_SIGNAL] = { 0 };
 
-
-GType
-gtk_entry_completion_get_type (void)
-{
-  static GType entry_completion_type = 0;
-
-  if (!entry_completion_type)
-    {
-      static const GTypeInfo entry_completion_info =
-      {
-        sizeof (GtkEntryCompletionClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) gtk_entry_completion_class_init,
-        NULL,
-        NULL,
-        sizeof (GtkEntryCompletion),
-        0,
-        (GInstanceInitFunc) gtk_entry_completion_init
-      };
-
-      static const GInterfaceInfo cell_layout_info =
-      {
-        (GInterfaceInitFunc) gtk_entry_completion_cell_layout_init,
-        NULL,
-        NULL
-      };
-
-      entry_completion_type =
-        g_type_register_static (G_TYPE_OBJECT, "GtkEntryCompletion",
-                                &entry_completion_info, 0);
-
-      g_type_add_interface_static (entry_completion_type,
-                                   GTK_TYPE_CELL_LAYOUT,
-                                   &cell_layout_info);
-    }
-
-  return entry_completion_type;
-}
+G_DEFINE_TYPE_WITH_CODE (GtkEntryCompletion, gtk_entry_completion, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_CELL_LAYOUT,
+						gtk_entry_completion_cell_layout_init));
 
 static void
 gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
 {
   GObjectClass *object_class;
 
-  parent_class = g_type_class_peek_parent (klass);
   object_class = (GObjectClass *)klass;
 
   object_class->set_property = gtk_entry_completion_set_property;
@@ -216,7 +176,7 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    * Since: 2.6
    */ 
   entry_completion_signals[INSERT_PREFIX] =
-    g_signal_new ("insert_prefix",
+    g_signal_new (I_("insert_prefix"),
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkEntryCompletionClass, insert_prefix),
@@ -241,7 +201,7 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    * Since: 2.4
    */ 
   entry_completion_signals[MATCH_SELECTED] =
-    g_signal_new ("match_selected",
+    g_signal_new (I_("match_selected"),
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkEntryCompletionClass, match_selected),
@@ -261,7 +221,7 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    * Since: 2.4
    */
   entry_completion_signals[ACTION_ACTIVATED] =
-    g_signal_new ("action_activated",
+    g_signal_new (I_("action_activated"),
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkEntryCompletionClass, action_activated),
@@ -612,7 +572,7 @@ gtk_entry_completion_finalize (GObject *object)
   if (completion->priv->popup_window)
     gtk_widget_destroy (completion->priv->popup_window);
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_entry_completion_parent_class)->finalize (object);
 }
 
 /* implement cell layout interface */
@@ -1347,6 +1307,8 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
                         NULL);
 
   height += vertical_separator;
+  
+  gtk_widget_realize (completion->priv->tree_view);
 
   if (items <= 0)
     gtk_widget_hide (completion->priv->scrolled_window);
@@ -1444,7 +1406,7 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
 
   toplevel = gtk_widget_get_toplevel (completion->priv->entry);
   if (GTK_IS_WINDOW (toplevel))
-    gtk_window_group_add_window (_gtk_window_get_group (GTK_WINDOW (toplevel)), 
+    gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (toplevel)), 
 				 GTK_WINDOW (completion->priv->popup_window));
 
   /* prevent the first row being focused */
@@ -1452,6 +1414,7 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
 
   gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW (completion->priv->tree_view)));
   gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW (completion->priv->action_view)));
+  
 
   gtk_widget_show (completion->priv->popup_window);
     
@@ -1485,7 +1448,7 @@ gtk_entry_completion_match_selected (GtkEntryCompletion *completion,
   gchar *str = NULL;
 
   gtk_tree_model_get (model, iter, completion->priv->text_column, &str, -1);
-  gtk_entry_set_text (GTK_ENTRY (completion->priv->entry), str);
+  gtk_entry_set_text (GTK_ENTRY (completion->priv->entry), str ? str : "");
   
   /* move cursor to the end */
   gtk_editable_set_position (GTK_EDITABLE (completion->priv->entry), -1);
@@ -1593,9 +1556,8 @@ gtk_entry_completion_insert_prefix (GtkEntryCompletion *completion)
   gboolean done;
   gchar *prefix;
 
-  if (completion->priv->insert_text_id > 0)
-    g_signal_handler_block (completion->priv->entry,
-			    completion->priv->insert_text_id);
+  g_signal_handler_block (completion->priv->entry,
+			  completion->priv->insert_text_id);
   prefix = gtk_entry_completion_compute_prefix (completion);
   if (prefix)
     {
@@ -1603,9 +1565,8 @@ gtk_entry_completion_insert_prefix (GtkEntryCompletion *completion)
 		     0, prefix, &done);
       g_free (prefix);
     }
-  if (completion->priv->insert_text_id > 0)
-    g_signal_handler_unblock (completion->priv->entry,
-			      completion->priv->insert_text_id);
+  g_signal_handler_unblock (completion->priv->entry,
+			    completion->priv->insert_text_id);
 }
 
 /**
