@@ -84,8 +84,6 @@ enum
 };
 
 /* Prototypes */
-static void gtk_font_button_init                   (GtkFontButton      *font_button);
-static void gtk_font_button_class_init             (GtkFontButtonClass *klass);
 static void gtk_font_button_finalize               (GObject            *object);
 static void gtk_font_button_get_property           (GObject            *object,
                                                     guint               param_id,
@@ -111,51 +109,19 @@ static GtkWidget *gtk_font_button_create_inside     (GtkFontButton     *gfs);
 static void gtk_font_button_label_use_font          (GtkFontButton     *gfs);
 static void gtk_font_button_update_font_info        (GtkFontButton     *gfs);
 
-static gpointer parent_class = NULL;
 static guint font_button_signals[LAST_SIGNAL] = { 0 };
 
-GType
-gtk_font_button_get_type (void)
-{
-  static GType font_button_type = 0;
-  
-  if (!font_button_type)
-    {
-      static const GTypeInfo font_button_info =
-      {
-        sizeof (GtkFontButtonClass),
-        NULL,           /* base_init */
-        NULL,           /* base_finalize */
-        (GClassInitFunc) gtk_font_button_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (GtkFontButton),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) gtk_font_button_init,
-      };
-      
-      font_button_type =
-        g_type_register_static (GTK_TYPE_BUTTON, "GtkFontButton",
-                                &font_button_info, 0);
-    }
-  
-  return font_button_type;
-}
-
+G_DEFINE_TYPE (GtkFontButton, gtk_font_button, GTK_TYPE_BUTTON);
 
 static void
 gtk_font_button_class_init (GtkFontButtonClass *klass)
 {
-  GtkObjectClass *object_class;
   GObjectClass *gobject_class;
   GtkButtonClass *button_class;
   
-  object_class = (GtkObjectClass *) klass;
   gobject_class = (GObjectClass *) klass;
   button_class = (GtkButtonClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->finalize = gtk_font_button_finalize;
   gobject_class->set_property = gtk_font_button_set_property;
   gobject_class->get_property = gtk_font_button_get_property;
@@ -199,7 +165,8 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
   /**
    * GtkFontButton:use-font:
    * 
-   * If this property is set to %TRUE, the label will be drawn in the selected font.
+   * If this property is set to %TRUE, the label will be drawn 
+   * in the selected font.
    *
    * Since: 2.4
    */
@@ -214,7 +181,8 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
   /**
    * GtkFontButton:use-size:
    * 
-   * If this property is set to %TRUE, the label will be drawn with the selected font size.
+   * If this property is set to %TRUE, the label will be drawn 
+   * with the selected font size.
    *
    * Since: 2.4
    */
@@ -229,8 +197,9 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
   /**
    * GtkFontButton:show-style:
    * 
-   * If this property is set to %TRUE, the name of the selected font style will be shown in the label. For
-   * a more WYSIWIG way to show the selected style, see the ::use-font property. 
+   * If this property is set to %TRUE, the name of the selected font style 
+   * will be shown in the label. For a more WYSIWYG way to show the selected 
+   * style, see the ::use-font property. 
    *
    * Since: 2.4
    */
@@ -244,8 +213,9 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
   /**
    * GtkFontButton:show-size:
    * 
-   * If this property is set to %TRUE, the selected font size will be shown in the label. For
-   * a more WYSIWIG way to show the selected size, see the ::use-size property. 
+   * If this property is set to %TRUE, the selected font size will be shown 
+   * in the label. For a more WYSIWYG way to show the selected size, see the 
+   * ::use-size property. 
    *
    * Since: 2.4
    */
@@ -261,12 +231,13 @@ gtk_font_button_class_init (GtkFontButtonClass *klass)
    * GtkFontButton::font-set:
    * @widget: the object which received the signal.
    * 
-   * The ::font-set signal is emitted when the user selects a font. When handling this signal,
-   * use gtk_font_button_get_font_name() to find out which font was just selected.
+   * The ::font-set signal is emitted when the user selects a font. When 
+   * handling this signal, use gtk_font_button_get_font_name() to find out
+   * which font was just selected.
    *
    * Since: 2.4
    */
-  font_button_signals[FONT_SET] = g_signal_new ("font-set",
+  font_button_signals[FONT_SET] = g_signal_new (I_("font-set"),
                                                 G_TYPE_FROM_CLASS (gobject_class),
                                                 G_SIGNAL_RUN_FIRST,
                                                 G_STRUCT_OFFSET (GtkFontButtonClass, font_set),
@@ -313,7 +284,7 @@ gtk_font_button_finalize (GObject *object)
   g_free (font_button->priv->title);
   font_button->priv->title = NULL;
   
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_font_button_parent_class)->finalize (object);
 }
 
 static void
@@ -920,8 +891,8 @@ gtk_font_button_update_font_info (GtkFontButton *font_button)
 
   if (font_button->priv->show_size) 
     {
-      gchar *size = g_strdup_printf ("%d", 
-                                     pango_font_description_get_size (desc) / PANGO_SCALE);
+      gchar *size = g_strdup_printf ("%g",
+                                     pango_font_description_get_size (desc) / (double)PANGO_SCALE);
       
       gtk_label_set_text (GTK_LABEL (font_button->priv->size_label), size);
       
