@@ -36,6 +36,8 @@
 #include "gtkmenubar.h"
 #include "gtkmenuitem.h"
 #include "gtkseparatormenuitem.h"
+#include "gtkprivate.h"
+#include "gtkintl.h"
 #include "gtkalias.h"
 
 #define MENU_ITEM_CLASS(w)  GTK_MENU_ITEM_CLASS (GTK_OBJECT (w)->klass)
@@ -49,8 +51,6 @@ enum {
 };
 
 
-static void gtk_menu_item_class_init     (GtkMenuItemClass *klass);
-static void gtk_menu_item_init           (GtkMenuItem      *menu_item);
 static void gtk_menu_item_destroy        (GtkObject        *object);
 static void gtk_menu_item_finalize       (GObject          *object);
 static void gtk_menu_item_size_request   (GtkWidget        *widget,
@@ -95,35 +95,9 @@ static gboolean gtk_menu_item_can_activate_accel (GtkWidget *widget,
 						  guint      signal_id);
 
 
-static GtkItemClass *parent_class;
 static guint menu_item_signals[LAST_SIGNAL] = { 0 };
 
-GType
-gtk_menu_item_get_type (void)
-{
-  static GType menu_item_type = 0;
-
-  if (!menu_item_type)
-    {
-      static const GTypeInfo menu_item_info =
-      {
-	sizeof (GtkMenuItemClass),
-	NULL,           /* base_init */
-	NULL,           /* base_finalize */
-	(GClassInitFunc) gtk_menu_item_class_init,
-	NULL,           /* class_finalize */
-	NULL,           /* class_data */
-	sizeof (GtkMenuItem),
-	16,             /* n_preallocs */
-	(GInstanceInitFunc) gtk_menu_item_init,
-      };
-
-      menu_item_type = g_type_register_static (GTK_TYPE_ITEM, "GtkMenuItem",
-					       &menu_item_info, 0);
-    }
-
-  return menu_item_type;
-}
+G_DEFINE_TYPE (GtkMenuItem, gtk_menu_item, GTK_TYPE_ITEM);
 
 static void
 gtk_menu_item_class_init (GtkMenuItemClass *klass)
@@ -134,8 +108,6 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
   GtkContainerClass *container_class = GTK_CONTAINER_CLASS (klass);
   GtkItemClass *item_class = GTK_ITEM_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->finalize = gtk_menu_item_finalize;
 
   object_class->destroy = gtk_menu_item_destroy;
@@ -166,7 +138,7 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
   klass->hide_on_activate = TRUE;
 
   menu_item_signals[ACTIVATE] =
-    g_signal_new ("activate",
+    g_signal_new (I_("activate"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GtkMenuItemClass, activate),
@@ -176,7 +148,7 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
   widget_class->activate_signal = menu_item_signals[ACTIVATE];
 
   menu_item_signals[ACTIVATE_ITEM] =
-    g_signal_new ("activate_item",
+    g_signal_new (I_("activate_item"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkMenuItemClass, activate_item),
@@ -185,7 +157,7 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
 		  G_TYPE_NONE, 0);
 
   menu_item_signals[TOGGLE_SIZE_REQUEST] =
-    g_signal_new ("toggle_size_request",
+    g_signal_new (I_("toggle_size_request"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkMenuItemClass, toggle_size_request),
@@ -195,7 +167,7 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
 		  G_TYPE_POINTER);
 
   menu_item_signals[TOGGLE_SIZE_ALLOCATE] =
-    g_signal_new ("toggle_size_allocate",
+    g_signal_new (I_("toggle_size_allocate"),
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
  		  G_STRUCT_OFFSET (GtkMenuItemClass, toggle_size_allocate),
@@ -205,40 +177,40 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
 		  G_TYPE_INT);
 
   gtk_widget_class_install_style_property_parser (widget_class,
-						  g_param_spec_enum ("selected_shadow_type",
+						  g_param_spec_enum ("selected-shadow-type",
 								     "Selected Shadow Type",
 								     "Shadow type when item is selected",
 								     GTK_TYPE_SHADOW_TYPE,
 								     GTK_SHADOW_NONE,
-								     G_PARAM_READABLE),
+								     GTK_PARAM_READABLE),
 						  gtk_rc_property_parse_enum);
 
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("horizontal_padding",
+					   g_param_spec_int ("horizontal-padding",
 							     "Horizontal Padding",
 							     "Padding to left and right of the menu item",
 							     0,
 							     G_MAXINT,
 							     3,
-							     G_PARAM_READABLE));
+							     GTK_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("toggle_spacing",
+					   g_param_spec_int ("toggle-spacing",
 							     "Icon Spacing",
 							     "Space between icon and label",
 							     0,
 							     G_MAXINT,
 							     5,
-							     G_PARAM_READABLE));
+							     GTK_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("arrow_spacing",
+					   g_param_spec_int ("arrow-spacing",
 							     "Arrow Spacing",
 							     "Space between label and arrow",
 							     0,
 							     G_MAXINT,
 							     10,
-							     G_PARAM_READABLE));
+							     GTK_PARAM_READABLE));
 }
 
 static void
@@ -324,7 +296,7 @@ gtk_menu_item_destroy (GtkObject *object)
   if (menu_item->submenu)
     gtk_widget_destroy (menu_item->submenu);
 
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);
+  GTK_OBJECT_CLASS (gtk_menu_item_parent_class)->destroy (object);
 }
 
 static void
@@ -334,7 +306,7 @@ gtk_menu_item_finalize (GObject *object)
 
   g_free (menu_item->accel_path);
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_menu_item_parent_class)->finalize (object);
 }
 
 static void
@@ -498,22 +470,42 @@ gtk_menu_item_size_request (GtkWidget      *widget,
   GtkBin *bin;
   guint accel_width;
   guint horizontal_padding;
+  GtkPackDirection pack_dir;
+  GtkPackDirection child_pack_dir;
 
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
   g_return_if_fail (requisition != NULL);
 
   gtk_widget_style_get (widget,
- 			"horizontal_padding", &horizontal_padding,
+ 			"horizontal-padding", &horizontal_padding,
 			NULL);
   
   bin = GTK_BIN (widget);
   menu_item = GTK_MENU_ITEM (widget);
 
+  if (GTK_IS_MENU_BAR (widget->parent))
+    {
+      pack_dir = gtk_menu_bar_get_pack_direction (GTK_MENU_BAR (widget->parent));
+      child_pack_dir = gtk_menu_bar_get_child_pack_direction (GTK_MENU_BAR (widget->parent));
+    }
+  else
+    {
+      pack_dir = GTK_PACK_DIRECTION_LTR;
+      child_pack_dir = GTK_PACK_DIRECTION_LTR;
+    }
+
   requisition->width = (GTK_CONTAINER (widget)->border_width +
-			widget->style->xthickness + horizontal_padding) * 2;
+			widget->style->xthickness) * 2;
   requisition->height = (GTK_CONTAINER (widget)->border_width +
 			 widget->style->ythickness) * 2;
 
+  if ((pack_dir == GTK_PACK_DIRECTION_LTR || pack_dir == GTK_PACK_DIRECTION_RTL) &&
+      (child_pack_dir == GTK_PACK_DIRECTION_LTR || child_pack_dir == GTK_PACK_DIRECTION_RTL))
+    requisition->width += 2 * horizontal_padding;
+  else if ((pack_dir == GTK_PACK_DIRECTION_TTB || pack_dir == GTK_PACK_DIRECTION_BTT) &&
+      (child_pack_dir == GTK_PACK_DIRECTION_TTB || child_pack_dir == GTK_PACK_DIRECTION_BTT))
+    requisition->height += 2 * horizontal_padding;
+
   if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
     {
       GtkRequisition child_requisition;
@@ -528,7 +520,7 @@ gtk_menu_item_size_request (GtkWidget      *widget,
 	  guint arrow_spacing;
 	  
 	  gtk_widget_style_get (widget,
-				"arrow_spacing", &arrow_spacing,
+				"arrow-spacing", &arrow_spacing,
 				NULL);
 
 	  requisition->width += child_requisition.height;
@@ -537,10 +529,20 @@ gtk_menu_item_size_request (GtkWidget      *widget,
 	  requisition->width = MAX (requisition->width, get_minimum_width (widget));
 	}
     }
-  else
+  else /* separator item */
     {
-      /* separator item */
-      requisition->height += 4;
+      gboolean wide_separators;
+      gint     separator_height;
+
+      gtk_widget_style_get (widget,
+                            "wide-separators",  &wide_separators,
+                            "separator-height", &separator_height,
+                            NULL);
+
+      if (wide_separators)
+        requisition->height += separator_height + widget->style->ythickness;
+      else
+        requisition->height += widget->style->ythickness * 2;
     }
 
   accel_width = 0;
@@ -558,6 +560,8 @@ gtk_menu_item_size_allocate (GtkWidget     *widget,
   GtkBin *bin;
   GtkAllocation child_allocation;
   GtkTextDirection direction;
+  GtkPackDirection pack_dir;
+  GtkPackDirection child_pack_dir;
 
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
   g_return_if_fail (allocation != NULL);
@@ -567,6 +571,17 @@ gtk_menu_item_size_allocate (GtkWidget     *widget,
   
   direction = gtk_widget_get_direction (widget);
 
+  if (GTK_IS_MENU_BAR (widget->parent))
+    {
+      pack_dir = gtk_menu_bar_get_pack_direction (GTK_MENU_BAR (widget->parent));
+      child_pack_dir = gtk_menu_bar_get_child_pack_direction (GTK_MENU_BAR (widget->parent));
+    }
+  else
+    {
+      pack_dir = GTK_PACK_DIRECTION_LTR;
+      child_pack_dir = GTK_PACK_DIRECTION_LTR;
+    }
+    
   widget->allocation = *allocation;
 
   if (bin->child)
@@ -575,19 +590,36 @@ gtk_menu_item_size_allocate (GtkWidget     *widget,
       guint horizontal_padding;
 
       gtk_widget_style_get (widget,
-			    "horizontal_padding", &horizontal_padding,
+			    "horizontal-padding", &horizontal_padding,
 			    NULL);
+
+      child_allocation.x = GTK_CONTAINER (widget)->border_width + widget->style->xthickness;
+      child_allocation.y = GTK_CONTAINER (widget)->border_width + widget->style->ythickness;
+
+      if ((pack_dir == GTK_PACK_DIRECTION_LTR || pack_dir == GTK_PACK_DIRECTION_RTL) &&
+	  (child_pack_dir == GTK_PACK_DIRECTION_LTR || child_pack_dir == GTK_PACK_DIRECTION_RTL))
+	child_allocation.x += horizontal_padding;
+      else if ((pack_dir == GTK_PACK_DIRECTION_TTB || pack_dir == GTK_PACK_DIRECTION_BTT) &&
+	       (child_pack_dir == GTK_PACK_DIRECTION_TTB || child_pack_dir == GTK_PACK_DIRECTION_BTT))
+	child_allocation.y += horizontal_padding;
       
-      child_allocation.x = (GTK_CONTAINER (widget)->border_width +
-                            widget->style->xthickness +
-			    horizontal_padding);
-      child_allocation.y = (GTK_CONTAINER (widget)->border_width +
-			    widget->style->ythickness);
       child_allocation.width = MAX (1, (gint)allocation->width - child_allocation.x * 2);
       child_allocation.height = MAX (1, (gint)allocation->height - child_allocation.y * 2);
-      if (direction == GTK_TEXT_DIR_LTR)
-	child_allocation.x += GTK_MENU_ITEM (widget)->toggle_size;
-      child_allocation.width -= GTK_MENU_ITEM (widget)->toggle_size;
+
+      if (child_pack_dir == GTK_PACK_DIRECTION_LTR ||
+	  child_pack_dir == GTK_PACK_DIRECTION_RTL)
+	{
+	  if ((direction == GTK_TEXT_DIR_LTR) == (child_pack_dir != GTK_PACK_DIRECTION_RTL))
+	    child_allocation.x += GTK_MENU_ITEM (widget)->toggle_size;
+	  child_allocation.width -= GTK_MENU_ITEM (widget)->toggle_size;
+	}
+      else
+	{
+	  if ((direction == GTK_TEXT_DIR_LTR) == (child_pack_dir != GTK_PACK_DIRECTION_BTT))
+	    child_allocation.y += GTK_MENU_ITEM (widget)->toggle_size;
+	  child_allocation.height -= GTK_MENU_ITEM (widget)->toggle_size;
+	}
+
       child_allocation.x += widget->allocation.x;
       child_allocation.y += widget->allocation.y;
 
@@ -655,8 +687,8 @@ gtk_menu_item_unrealize (GtkWidget *widget)
   gdk_window_destroy (menu_item->event_window);
   menu_item->event_window = NULL;
   
-  if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-    (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
+  if (GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->unrealize)
+    (* GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->unrealize) (widget);
 }
 
 static void
@@ -664,7 +696,7 @@ gtk_menu_item_map (GtkWidget *widget)
 {
   GtkMenuItem *menu_item = GTK_MENU_ITEM (widget);
   
-  GTK_WIDGET_CLASS (parent_class)->map (widget);
+  GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->map (widget);
 
   gdk_window_show (menu_item->event_window);
 }
@@ -676,7 +708,7 @@ gtk_menu_item_unmap (GtkWidget *widget)
     
   gdk_window_hide (menu_item->event_window);
 
-  GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+  GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->unmap (widget);
 }
 
 static void
@@ -705,7 +737,7 @@ gtk_menu_item_paint (GtkWidget    *widget,
 	  (GTK_BIN (menu_item)->child))
 	{
 	  gtk_widget_style_get (widget,
-				"selected_shadow_type", &selected_shadow_type,
+				"selected-shadow-type", &selected_shadow_type,
 				NULL);
 	  gtk_paint_box (widget->style,
 			 widget->window,
@@ -730,7 +762,7 @@ gtk_menu_item_paint (GtkWidget    *widget,
 	  direction = gtk_widget_get_direction (widget);
       
  	  gtk_widget_style_get (widget,
- 				"horizontal_padding", &horizontal_padding,
+ 				"horizontal-padding", &horizontal_padding,
  				NULL);
  	  
 	  context = gtk_widget_get_pango_context (GTK_BIN (menu_item)->child);
@@ -772,18 +804,34 @@ gtk_menu_item_paint (GtkWidget    *widget,
 	}
       else if (!GTK_BIN (menu_item)->child)
 	{
-	  guint horizontal_padding;
+          gboolean wide_separators;
+          gint     separator_height;
+	  guint    horizontal_padding;
 
 	  gtk_widget_style_get (widget,
-				"horizontal_padding", &horizontal_padding,
-				NULL);
-	  
-	  gtk_paint_hline (widget->style, widget->window, GTK_STATE_NORMAL,
-			   area, widget, "menuitem",
-			   widget->allocation.x + horizontal_padding + widget->style->xthickness,
-			   widget->allocation.x + widget->allocation.width - horizontal_padding - widget->style->xthickness - 1,
-			   widget->allocation.y + (widget->allocation.height -
-						   widget->style->ythickness) / 2);
+                                "wide-separators",    &wide_separators,
+                                "separator-height",   &separator_height,
+                                "horizontal-padding", &horizontal_padding,
+                                NULL);
+
+          if (wide_separators)
+            gtk_paint_box (widget->style, widget->window,
+                           GTK_STATE_NORMAL, GTK_SHADOW_ETCHED_OUT,
+                           area, widget, "hseparator",
+                           widget->allocation.x + horizontal_padding + widget->style->xthickness,
+                           widget->allocation.y + (widget->allocation.height -
+                                                   separator_height -
+                                                   widget->style->ythickness) / 2,
+                           widget->allocation.width -
+                           2 * (horizontal_padding + widget->style->xthickness),
+                           separator_height);
+          else
+            gtk_paint_hline (widget->style, widget->window,
+                             GTK_STATE_NORMAL, area, widget, "menuitem",
+                             widget->allocation.x + horizontal_padding + widget->style->xthickness,
+                             widget->allocation.x + widget->allocation.width - horizontal_padding - widget->style->xthickness - 1,
+                             widget->allocation.y + (widget->allocation.height -
+                                                     widget->style->ythickness) / 2);
 	}
     }
 }
@@ -799,7 +847,7 @@ gtk_menu_item_expose (GtkWidget      *widget,
     {
       gtk_menu_item_paint (widget, &event->area);
 
-      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
+      (* GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->expose_event) (widget, event);
     }
 
   return FALSE;
@@ -922,7 +970,6 @@ static void
 gtk_real_menu_item_activate_item (GtkMenuItem *menu_item)
 {
   GtkWidget *widget;
-  GtkMenuShell *submenu; 
 
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
@@ -944,7 +991,6 @@ gtk_real_menu_item_activate_item (GtkMenuItem *menu_item)
 	  _gtk_menu_item_popup_submenu (widget); 
 
 	  gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_item->submenu), TRUE);
-	  submenu = GTK_MENU_SHELL (menu_item->submenu);
 	}
     }
 }
@@ -1003,13 +1049,21 @@ _gtk_menu_item_popup_submenu (GtkWidget *widget)
   menu_item->timer = 0;
 
   if (GTK_WIDGET_IS_SENSITIVE (menu_item->submenu))
-    gtk_menu_popup (GTK_MENU (menu_item->submenu),
-		    widget->parent,
-		    widget,
-		    gtk_menu_item_position_menu,
-		    menu_item,
-		    GTK_MENU_SHELL (widget->parent)->button,
-		    0);
+    {
+      gboolean take_focus;
+
+      take_focus = gtk_menu_shell_get_take_focus (GTK_MENU_SHELL (widget->parent));
+      gtk_menu_shell_set_take_focus (GTK_MENU_SHELL (menu_item->submenu),
+                                     take_focus);
+
+      gtk_menu_popup (GTK_MENU (menu_item->submenu),
+                      widget->parent,
+                      widget,
+                      gtk_menu_item_position_menu,
+                      menu_item,
+                      GTK_MENU_SHELL (widget->parent)->button,
+                      0);
+    }
 }
 
 static void
@@ -1020,9 +1074,9 @@ get_offsets (GtkMenu *menu,
   gint vertical_padding;
   
   gtk_widget_style_get (GTK_WIDGET (menu),
-			"horizontal_offset", horizontal_offset,
-			"vertical_offset", vertical_offset,
-			"vertical_padding", &vertical_padding,
+			"horizontal-offset", horizontal_offset,
+			"vertical-offset", vertical_offset,
+			"vertical-padding", &vertical_padding,
 			NULL);
 
   *vertical_offset -= GTK_WIDGET (menu)->style->ythickness;
@@ -1038,7 +1092,7 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
 {
   GtkMenuItem *menu_item;
   GtkWidget *widget;
-  GtkWidget *parent_menu_item;
+  GtkMenuItem *parent_menu_item;
   GdkScreen *screen;
   gint twidth, theight;
   gint tx, ty;
@@ -1080,6 +1134,22 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   ty += widget->allocation.y;
 
   get_offsets (menu, &horizontal_offset, &vertical_offset);
+
+  if (GTK_IS_MENU_BAR (widget->parent))
+    {
+      menu_item->from_menubar = TRUE;
+    }
+  else if (GTK_IS_MENU (widget->parent))
+    {
+      if (GTK_MENU (widget->parent)->parent_menu_item)
+	menu_item->from_menubar = GTK_MENU_ITEM (GTK_MENU (widget->parent)->parent_menu_item)->from_menubar;
+      else
+	menu_item->from_menubar = FALSE;
+    }
+  else
+    {
+      menu_item->from_menubar = FALSE;
+    }
   
   switch (menu_item->submenu_placement)
     {
@@ -1091,7 +1161,6 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
 	  menu_item->submenu_direction = GTK_DIRECTION_LEFT;
 	  tx += widget->allocation.width - twidth;
 	}
-
       if ((ty + widget->allocation.height + theight) <= monitor.y + monitor.height)
 	ty += widget->allocation.height;
       else if ((ty - theight) >= monitor.y)
@@ -1103,14 +1172,24 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
       break;
 
     case GTK_LEFT_RIGHT:
-      parent_menu_item = GTK_MENU (widget->parent)->parent_menu_item;
-      parent_xthickness = widget->parent->style->xthickness;
-      if (parent_menu_item && !GTK_MENU (widget->parent)->torn_off)
-	menu_item->submenu_direction = GTK_MENU_ITEM (parent_menu_item)->submenu_direction;
-      else if (direction == GTK_TEXT_DIR_LTR)
-	menu_item->submenu_direction = GTK_DIRECTION_RIGHT;
+      if (GTK_IS_MENU (widget->parent))
+	parent_menu_item = GTK_MENU_ITEM (GTK_MENU (widget->parent)->parent_menu_item);
       else
-	menu_item->submenu_direction = GTK_DIRECTION_LEFT;
+	parent_menu_item = NULL;
+      
+      parent_xthickness = widget->parent->style->xthickness;
+
+      if (parent_menu_item && !GTK_MENU (widget->parent)->torn_off)
+	{
+	  menu_item->submenu_direction = parent_menu_item->submenu_direction;
+	}
+      else
+	{
+	  if (direction == GTK_TEXT_DIR_LTR)
+	    menu_item->submenu_direction = GTK_DIRECTION_RIGHT;
+	  else
+	    menu_item->submenu_direction = GTK_DIRECTION_LEFT;
+	}
 
       switch (menu_item->submenu_direction)
 	{
@@ -1149,6 +1228,12 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   *y = ty;
 
   gtk_menu_set_monitor (menu, monitor_num);
+
+  if (!GTK_WIDGET_VISIBLE (menu->toplevel))
+    {
+      gtk_window_set_type_hint (GTK_WINDOW (menu->toplevel), menu_item->from_menubar?
+				GDK_WINDOW_TYPE_HINT_DROPDOWN_MENU : GDK_WINDOW_TYPE_HINT_POPUP_MENU);
+    }
 }
 
 /**
@@ -1274,8 +1359,8 @@ gtk_menu_item_parent_set (GtkWidget *widget,
 				       menu->accel_group,
 				       TRUE);
 
-  if (GTK_WIDGET_CLASS (parent_class)->parent_set)
-    GTK_WIDGET_CLASS (parent_class)->parent_set (widget, previous_parent);
+  if (GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->parent_set)
+    GTK_WIDGET_CLASS (gtk_menu_item_parent_class)->parent_set (widget, previous_parent);
 }
 
 void
@@ -1380,13 +1465,11 @@ gtk_menu_item_forall (GtkContainer *container,
 		      gpointer      callback_data)
 {
   GtkBin *bin;
-  GtkMenuItem *menu_item;
 
   g_return_if_fail (GTK_IS_MENU_ITEM (container));
   g_return_if_fail (callback != NULL);
 
   bin = GTK_BIN (container);
-  menu_item = GTK_MENU_ITEM (container);
 
   if (bin->child)
     callback (bin->child, callback_data);
