@@ -45,6 +45,8 @@ struct _GtkPathBar
   GtkFilePath *home_path;
   GtkFilePath *desktop_path;
 
+  GtkFileSystemHandle *set_path_handle;
+
   GdkPixbuf *root_icon;
   GdkPixbuf *home_icon;
   GdkPixbuf *desktop_icon;
@@ -61,8 +63,10 @@ struct _GtkPathBar
   gint16 button_offset;
   guint timer;
   guint slider_visible : 1;
-  guint need_timer : 1;
-  guint ignore_click : 1;
+  guint need_timer     : 1;
+  guint ignore_click   : 1;
+  guint scrolling_up   : 1;
+  guint scrolling_down : 1;
 };
 
 struct _GtkPathBarClass
@@ -71,6 +75,7 @@ struct _GtkPathBarClass
 
   void (* path_clicked) (GtkPathBar  *path_bar,
 			 GtkFilePath *file_path,
+			 GtkFilePath *child_path,
 			 gboolean     child_is_hidden);
 };
 
@@ -79,6 +84,7 @@ void     _gtk_path_bar_set_file_system (GtkPathBar         *path_bar,
 					GtkFileSystem      *file_system);
 gboolean _gtk_path_bar_set_path        (GtkPathBar         *path_bar,
 					const GtkFilePath  *file_path,
+					gboolean            keep_trail,
 					GError            **error);
 void     _gtk_path_bar_up              (GtkPathBar *path_bar);
 void     _gtk_path_bar_down            (GtkPathBar *path_bar);
