@@ -45,8 +45,6 @@ enum {
 };
 
 
-static void gtk_progress_class_init      (GtkProgressClass *klass);
-static void gtk_progress_init            (GtkProgress      *progress);
 static void gtk_progress_set_property    (GObject          *object,
 					  guint             prop_id,
 					  const GValue     *value,
@@ -68,36 +66,7 @@ static void gtk_progress_value_changed   (GtkAdjustment    *adjustment,
 static void gtk_progress_changed         (GtkAdjustment    *adjustment,
 					  GtkProgress      *progress);
 
-
-static GtkWidgetClass *parent_class = NULL;
-
-
-GType
-gtk_progress_get_type (void)
-{
-  static GType progress_type = 0;
-
-  if (!progress_type)
-    {
-      static const GTypeInfo progress_info =
-      {
-	sizeof (GtkProgressClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_progress_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkProgress),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_progress_init,
-      };
-
-      progress_type = g_type_register_static (GTK_TYPE_WIDGET, "GtkProgress",
-					      &progress_info, 0);
-    }
-
-  return progress_type;
-}
+G_DEFINE_TYPE (GtkProgress, gtk_progress, GTK_TYPE_WIDGET);
 
 static void
 gtk_progress_class_init (GtkProgressClass *class)
@@ -109,8 +78,6 @@ gtk_progress_class_init (GtkProgressClass *class)
   object_class = (GtkObjectClass *) class;
   widget_class = (GtkWidgetClass *) class;
 
-  parent_class = gtk_type_class (GTK_TYPE_WIDGET);
-
   gobject_class->finalize = gtk_progress_finalize;
 
   gobject_class->set_property = gtk_progress_set_property;
@@ -295,7 +262,7 @@ gtk_progress_destroy (GtkObject *object)
       progress->adjustment = NULL;
     }
 
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);
+  GTK_OBJECT_CLASS (gtk_progress_parent_class)->destroy (object);
 }
 
 static void
@@ -313,7 +280,7 @@ gtk_progress_finalize (GObject *object)
   if (progress->format)
     g_free (progress->format);
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_progress_parent_class)->finalize (object);
 }
 
 static gint
@@ -372,6 +339,13 @@ gtk_progress_create_pixmap (GtkProgress *progress)
 						   widget->allocation.width,
 						   widget->allocation.height,
 						   -1);
+
+      /* clear the pixmap for transparent themes */
+      gtk_paint_flat_box (widget->style,
+                          progress->offscreen_pixmap,
+                          GTK_STATE_NORMAL, GTK_SHADOW_NONE,
+                          NULL, widget, "trough", 0, 0, -1, -1);
+      
       GTK_PROGRESS_GET_CLASS (progress)->paint (progress);
     }
 }
@@ -535,8 +509,7 @@ gtk_progress_set_adjustment (GtkProgress   *progress,
       progress->adjustment = adjustment;
       if (adjustment)
         {
-          g_object_ref (adjustment);
-	  gtk_object_sink (GTK_OBJECT (adjustment));
+          g_object_ref_sink (adjustment);
           g_signal_connect (adjustment, "changed",
 			    G_CALLBACK (gtk_progress_changed),
 			    progress);
