@@ -65,8 +65,6 @@ enum {
   PROP_DISCRETE_BLOCKS
 };
 
-static void gtk_progress_bar_class_init    (GtkProgressBarClass *klass);
-static void gtk_progress_bar_init          (GtkProgressBar      *pbar);
 static void gtk_progress_bar_set_property  (GObject             *object,
 					    guint                prop_id,
 					    const GValue        *value,
@@ -91,33 +89,7 @@ static void gtk_progress_bar_set_activity_blocks_internal (GtkProgressBar *pbar,
 							   guint           blocks);
 
 
-GType
-gtk_progress_bar_get_type (void)
-{
-  static GType progress_bar_type = 0;
-
-  if (!progress_bar_type)
-    {
-      static const GTypeInfo progress_bar_info =
-      {
-	sizeof (GtkProgressBarClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_progress_bar_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkProgressBar),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_progress_bar_init,
-      };
-
-      progress_bar_type =
-	g_type_register_static (GTK_TYPE_PROGRESS, "GtkProgressBar",
-				&progress_bar_info, 0);
-    }
-
-  return progress_bar_type;
-}
+G_DEFINE_TYPE (GtkProgressBar, gtk_progress_bar, GTK_TYPE_PROGRESS);
 
 static void
 gtk_progress_bar_class_init (GtkProgressBarClass *class)
@@ -543,29 +515,13 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
   if (pbar->orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
       pbar->orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
     {
-      if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
-	{
-	  requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH, width);
-	  requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT, height);
-	}
-      else
-	{
-	  requisition->width = MIN_HORIZONTAL_BAR_WIDTH;
-	  requisition->height = MIN_HORIZONTAL_BAR_HEIGHT;
-	}
+      requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH, width);
+      requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT, height);
     }
   else
     {
-      if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
-	{	  
-	  requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH, width);
-	  requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT, height);
-	}
-      else
-	{
-	  requisition->width = MIN_VERTICAL_BAR_WIDTH;
-	  requisition->height = MIN_VERTICAL_BAR_HEIGHT;
-	}
+      requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH, width);
+      requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT, height);
     }
 }
 
@@ -574,7 +530,6 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
 {
   GtkProgressBar *pbar;
   GtkWidget *widget;
-  gint size;
   GtkProgressBarOrientation orientation;
 
   pbar = GTK_PROGRESS_BAR (progress);
@@ -594,8 +549,6 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
   if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
       orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
     {
-      size = MAX (2, widget->allocation.width / pbar->activity_blocks);
-
       if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT)
 	{
 	  pbar->activity_pos = widget->style->xthickness;
@@ -611,8 +564,6 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
     }
   else
     {
-      size = MAX (2, widget->allocation.height / pbar->activity_blocks);
-
       if (orientation == GTK_PROGRESS_TOP_TO_BOTTOM)
 	{
 	  pbar->activity_pos = widget->style->ythickness;
@@ -634,24 +585,24 @@ gtk_progress_bar_paint_activity (GtkProgressBar            *pbar,
 {
   GtkWidget *widget = GTK_WIDGET (pbar);
   GtkProgress *progress = GTK_PROGRESS (pbar);
-  gint x, y, w, h;
+  GdkRectangle area;
 
   switch (orientation)
     {
     case GTK_PROGRESS_LEFT_TO_RIGHT:
     case GTK_PROGRESS_RIGHT_TO_LEFT:
-      x = pbar->activity_pos;
-      y = widget->style->ythickness;
-      w = MAX (2, widget->allocation.width / pbar->activity_blocks);
-      h = widget->allocation.height - 2 * widget->style->ythickness;
+      area.x = pbar->activity_pos;
+      area.y = widget->style->ythickness;
+      area.width = MAX (2, widget->allocation.width / pbar->activity_blocks);
+      area.height = widget->allocation.height - 2 * widget->style->ythickness;
       break;
 
     case GTK_PROGRESS_TOP_TO_BOTTOM:
     case GTK_PROGRESS_BOTTOM_TO_TOP:
-      x = widget->style->xthickness;
-      y = pbar->activity_pos;
-      w = widget->allocation.width - 2 * widget->style->xthickness;
-      h = MAX (2, widget->allocation.height / pbar->activity_blocks);
+      area.x = widget->style->xthickness;
+      area.y = pbar->activity_pos;
+      area.width = widget->allocation.width - 2 * widget->style->xthickness;
+      area.height = MAX (2, widget->allocation.height / pbar->activity_blocks);
       break;
 
     default:
@@ -662,8 +613,8 @@ gtk_progress_bar_paint_activity (GtkProgressBar            *pbar,
   gtk_paint_box (widget->style,
 		 progress->offscreen_pixmap,
 		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		 NULL, widget, "bar",
-		 x, y, w, h);
+		 &area, widget, "bar",
+		 area.x, area.y, area.width, area.height);
 }
 
 static void
@@ -671,8 +622,8 @@ gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
 				   gint                       amount,
 				   GtkProgressBarOrientation  orientation)
 {
+  GdkRectangle area;
   GtkWidget *widget = GTK_WIDGET (pbar);
-  gint x, y, w, h;
 
   if (amount <= 0)
     return;
@@ -681,24 +632,24 @@ gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
     {
     case GTK_PROGRESS_LEFT_TO_RIGHT:
     case GTK_PROGRESS_RIGHT_TO_LEFT:
-      w = amount;
-      h = widget->allocation.height - widget->style->ythickness * 2;
-      y = widget->style->ythickness;
+      area.width = amount;
+      area.height = widget->allocation.height - widget->style->ythickness * 2;
+      area.y = widget->style->ythickness;
       
-      x = widget->style->xthickness;
+      area.x = widget->style->xthickness;
       if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
-	x = widget->allocation.width - amount - x;
+	area.x = widget->allocation.width - amount - area.x;
       break;
       
     case GTK_PROGRESS_TOP_TO_BOTTOM:
     case GTK_PROGRESS_BOTTOM_TO_TOP:
-      w = widget->allocation.width - widget->style->xthickness * 2;
-      h = amount;
-      x = widget->style->xthickness;
+      area.width = widget->allocation.width - widget->style->xthickness * 2;
+      area.height = amount;
+      area.x = widget->style->xthickness;
       
-      y = widget->style->ythickness;
+      area.y = widget->style->ythickness;
       if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
-	y = widget->allocation.height - amount - y;
+	area.y = widget->allocation.height - amount - area.y;
       break;
       
     default:
@@ -709,8 +660,8 @@ gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
   gtk_paint_box (widget->style,
 		 GTK_PROGRESS (pbar)->offscreen_pixmap,
 		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		 NULL, widget, "bar",
-		 x, y, w, h);
+		 &area, widget, "bar",
+		 area.x, area.y, area.width, area.height);
 }
 
 static void
@@ -722,7 +673,8 @@ gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
 
   for (i = 0; i <= pbar->in_block; i++)
     {
-      gint x, y, w, h, space;
+      GdkRectangle area;
+      gint space;
 
       switch (orientation)
 	{
@@ -730,26 +682,26 @@ gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
 	case GTK_PROGRESS_RIGHT_TO_LEFT:
 	  space = widget->allocation.width - 2 * widget->style->xthickness;
 	  
-	  x = widget->style->xthickness + (i * space) / pbar->blocks;
-	  y = widget->style->ythickness;
-	  w = widget->style->xthickness + ((i + 1) * space) / pbar->blocks - x;
-	  h = widget->allocation.height - 2 * widget->style->ythickness;
+	  area.x = widget->style->xthickness + (i * space) / pbar->blocks;
+	  area.y = widget->style->ythickness;
+	  area.width = widget->style->xthickness + ((i + 1) * space) / pbar->blocks - area.x;
+	  area.height = widget->allocation.height - 2 * widget->style->ythickness;
 
 	  if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
-	    x = widget->allocation.width - w - x;
+	    area.x = widget->allocation.width - area.width - area.x;
 	  break;
 	  
 	case GTK_PROGRESS_TOP_TO_BOTTOM:
 	case GTK_PROGRESS_BOTTOM_TO_TOP:
 	  space = widget->allocation.height - 2 * widget->style->ythickness;
 	  
-	  x = widget->style->xthickness;
-	  y = widget->style->ythickness + (i * space) / pbar->blocks;
-	  w = widget->allocation.width - 2 * widget->style->xthickness;
-	  h = widget->style->ythickness + ((i + 1) * space) / pbar->blocks - y;
+	  area.x = widget->style->xthickness;
+	  area.y = widget->style->ythickness + (i * space) / pbar->blocks;
+	  area.width = widget->allocation.width - 2 * widget->style->xthickness;
+	  area.height = widget->style->ythickness + ((i + 1) * space) / pbar->blocks - area.y;
 	  
 	  if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
-	    y = widget->allocation.height - h - y;
+	    area.y = widget->allocation.height - area.height - area.y;
 	  break;
 
 	default:
@@ -760,8 +712,8 @@ gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
       gtk_paint_box (widget->style,
 		     GTK_PROGRESS (pbar)->offscreen_pixmap,
 		     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		     NULL, widget, "bar",
-		     x, y, w, h);
+		     &area, widget, "bar",
+		     area.x, area.y, area.width, area.height);
     }
 }
 
@@ -1038,7 +990,7 @@ gtk_progress_bar_pulse (GtkProgressBar *pbar)
 /**
  * gtk_progress_bar_set_text:
  * @pbar: a #GtkProgressBar
- * @text: a UTF-8 string
+ * @text: a UTF-8 string, or %NULL 
  * 
  * Causes the given @text to appear superimposed on the progress bar.
  **/
