@@ -19,15 +19,16 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkcontainer.h"
 #include "gtkintl.h"
 #include "gtkprivate.h"
 #include "gtksizegroup.h"
+#include "gtkalias.h"
 
 enum {
   PROP_0,
-  PROP_MODE
+  PROP_MODE,
+  PROP_IGNORE_HIDDEN
 };
 
 static void gtk_size_group_set_property (GObject      *object,
@@ -51,12 +52,12 @@ static void add_widget_to_closure (GtkWidget         *widget,
 static GQuark size_groups_quark;
 static const gchar size_groups_tag[] = "gtk-size-groups";
 
+static GQuark visited_quark;
+static const gchar visited_tag[] = "gtk-size-group-visited";
+
 static GSList *
 get_size_groups (GtkWidget *widget)
 {
-  if (!size_groups_quark)
-    size_groups_quark = g_quark_from_string (size_groups_tag);
-  
   return g_object_get_qdata (G_OBJECT (widget), size_groups_quark);
 }
 
@@ -64,12 +65,27 @@ static void
 set_size_groups (GtkWidget *widget,
 		 GSList    *groups)
 {
-  if (!size_groups_quark)
-    size_groups_quark = g_quark_from_string (size_groups_tag);
-
   g_object_set_qdata (G_OBJECT (widget), size_groups_quark, groups);
 }
 
+static void
+mark_visited (gpointer object)
+{
+  g_object_set_qdata (object, visited_quark, "visited");
+}
+
+static void
+mark_unvisited (gpointer object)
+{
+  g_object_set_qdata (object, visited_quark, NULL);
+}
+
+static gboolean
+is_visited (gpointer object)
+{
+  return g_object_get_qdata (object, visited_quark) != NULL;
+}
+
 static void
 add_group_to_closure (GtkSizeGroup    *group,
 		      GtkSizeGroupMode mode,
@@ -79,13 +95,14 @@ add_group_to_closure (GtkSizeGroup    *group,
   GSList *tmp_widgets;
   
   *groups = g_slist_prepend (*groups, group);
+  mark_visited (group);
 
   tmp_widgets = group->widgets;
   while (tmp_widgets)
     {
       GtkWidget *tmp_widget = tmp_widgets->data;
       
-      if (!g_slist_find (*widgets, tmp_widget))
+      if (!is_visited (tmp_widget))
 	add_widget_to_closure (tmp_widget, mode, groups, widgets);
       
       tmp_widgets = tmp_widgets->next;
@@ -101,6 +118,7 @@ add_widget_to_closure (GtkWidget       *widget,
   GSList *tmp_groups;
 
   *widgets = g_slist_prepend (*widgets, widget);
+  mark_visited (widget);
 
   tmp_groups = get_size_groups (widget);
   while (tmp_groups)
@@ -108,7 +126,7 @@ add_widget_to_closure (GtkWidget       *widget,
       GtkSizeGroup *tmp_group = tmp_groups->data;
       
       if ((tmp_group->mode == GTK_SIZE_GROUP_BOTH || tmp_group->mode == mode) &&
-	  !g_slist_find (*groups, tmp_group))
+	  !is_visited (tmp_group))
 	add_group_to_closure (tmp_group, mode, groups, widgets);
 
       tmp_groups = tmp_groups->next;
@@ -176,6 +194,9 @@ queue_resize_on_widget (GtkWidget *widget,
       widgets = NULL;
 	  
       add_widget_to_closure (parent, GTK_SIZE_GROUP_HORIZONTAL, &groups, &widgets);
+      g_slist_foreach (widgets, (GFunc)mark_unvisited, NULL);
+      g_slist_foreach (groups, (GFunc)mark_unvisited, NULL);
+
       reset_group_sizes (groups);
 	      
       tmp_list = widgets;
@@ -199,6 +220,9 @@ queue_resize_on_widget (GtkWidget *widget,
       widgets = NULL;
 	      
       add_widget_to_closure (parent, GTK_SIZE_GROUP_VERTICAL, &groups, &widgets);
+      g_slist_foreach (widgets, (GFunc)mark_unvisited, NULL);
+      g_slist_foreach (groups, (GFunc)mark_unvisited, NULL);
+
       reset_group_sizes (groups);
 	      
       tmp_list = widgets;
@@ -241,11 +265,30 @@ gtk_size_group_class_init (GtkSizeGroupClass *klass)
 				   PROP_MODE,
 				   g_param_spec_enum ("mode",
 						      P_("Mode"),
-						      P_("The directions in which the size group effects the requested sizes"
+						      P_("The directions in which the size group affects the requested sizes"
 							" of its component widgets"),
 						      GTK_TYPE_SIZE_GROUP_MODE,
-						      GTK_SIZE_GROUP_HORIZONTAL,
-						      G_PARAM_READWRITE));
+						      GTK_SIZE_GROUP_HORIZONTAL,						      GTK_PARAM_READWRITE));
+
+  /**
+   * GtkSizeGroup:ignore-hidden:
+   *
+   * If %TRUE, hidden widgets are ignored when determining 
+   * the size of the group.
+   *
+   * Since: 2.8
+   */
+  g_object_class_install_property (gobject_class,
+				   PROP_IGNORE_HIDDEN,
+				   g_param_spec_boolean ("ignore-hidden",
+							 P_("Ignore hidden"),
+							 P_("If TRUE, hidden widgets are ignored "
+							    "when determining the size of the group"),
+							 FALSE,
+							 GTK_PARAM_READWRITE));
+
+  size_groups_quark = g_quark_from_static_string (size_groups_tag);
+  visited_quark = g_quark_from_string (visited_tag);
 }
 
 static void
@@ -255,34 +298,10 @@ gtk_size_group_init (GtkSizeGroup *size_group)
   size_group->mode = GTK_SIZE_GROUP_HORIZONTAL;
   size_group->have_width = 0;
   size_group->have_height = 0;
+  size_group->ignore_hidden = 0;
 }
 
-GType
-gtk_size_group_get_type (void)
-{
-  static GType size_group_type = 0;
-
-  if (!size_group_type)
-    {
-      static const GTypeInfo size_group_info =
-      {
-	sizeof (GtkSizeGroupClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_size_group_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkSizeGroup),
-	16,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_size_group_init,
-      };
-
-      size_group_type = g_type_register_static (G_TYPE_OBJECT, "GtkSizeGroup",
-						&size_group_info, 0);
-    }
-
-  return size_group_type;
-}
+G_DEFINE_TYPE (GtkSizeGroup, gtk_size_group, G_TYPE_OBJECT);
 
 static void
 gtk_size_group_set_property (GObject      *object,
@@ -297,6 +316,9 @@ gtk_size_group_set_property (GObject      *object,
     case PROP_MODE:
       gtk_size_group_set_mode (size_group, g_value_get_enum (value));
       break;
+    case PROP_IGNORE_HIDDEN:
+      gtk_size_group_set_ignore_hidden (size_group, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -316,6 +338,9 @@ gtk_size_group_get_property (GObject      *object,
     case PROP_MODE:
       g_value_set_enum (value, size_group->mode);
       break;
+    case PROP_IGNORE_HIDDEN:
+      g_value_set_boolean (value, size_group->ignore_hidden);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -386,6 +411,51 @@ gtk_size_group_get_mode (GtkSizeGroup *size_group)
   return size_group->mode;
 }
 
+/**
+ * gtk_size_group_set_ignore_hidden:
+ * @size_group: a #GtkSizeGroup
+ * @ignore_hidden: whether hidden widgets should be ignored
+ *   when calculating the size
+ * 
+ * Sets whether invisible widgets should be ignored when
+ * calculating the size.
+ *
+ * Since: 2.8 
+ */
+void
+gtk_size_group_set_ignore_hidden (GtkSizeGroup *size_group,
+				  gboolean      ignore_hidden)
+{
+  g_return_if_fail (GTK_IS_SIZE_GROUP (size_group));
+  
+  ignore_hidden = ignore_hidden != FALSE;
+
+  if (size_group->ignore_hidden != ignore_hidden)
+    {
+      size_group->ignore_hidden = ignore_hidden;
+
+      g_object_notify (G_OBJECT (size_group), "ignore-hidden");
+    }
+}
+
+/**
+ * gtk_size_group_get_ignore_hidden:
+ * @size_group: a #GtkSizeGroup
+ *
+ * Returns if invisible widgets are ignored when calculating the size.
+ *
+ * Returns: %TRUE if invisible widgets are ignored.
+ *
+ * Since: 2.8
+ */
+gboolean
+gtk_size_group_get_ignore_hidden (GtkSizeGroup *size_group)
+{
+  g_return_val_if_fail (GTK_IS_SIZE_GROUP (size_group), FALSE);
+
+  return size_group->ignore_hidden;
+}
+
 static void
 gtk_size_group_widget_destroyed (GtkWidget    *widget,
 				 GtkSizeGroup *size_group)
@@ -464,6 +534,23 @@ gtk_size_group_remove_widget (GtkSizeGroup     *size_group,
   g_object_unref (size_group);
 }
 
+/**
+ * gtk_size_group_get_widgets:
+ * @size_group: a #GtkSizeGrup
+ * 
+ * Returns the list of widgets associated with @size_group.
+ *
+ * Return value: a #GSList of widgets. The list is owned by GTK+ 
+ *   and should not be modified.
+ *
+ * Since: 2.10
+ **/
+GSList *
+gtk_size_group_get_widgets (GtkSizeGroup *size_group)
+{
+     return size_group->widgets;
+}
+
 static gint
 get_base_dimension (GtkWidget        *widget,
 		    GtkSizeGroupMode  mode)
@@ -491,12 +578,11 @@ do_size_request (GtkWidget *widget)
 {
   if (GTK_WIDGET_REQUEST_NEEDED (widget))
     {
-      gtk_widget_ensure_style (widget);
+      gtk_widget_ensure_style (widget);      
+      GTK_PRIVATE_UNSET_FLAG (widget, GTK_REQUEST_NEEDED);
       g_signal_emit_by_name (widget,
 			     "size_request",
 			     &widget->requisition);
-      
-      GTK_PRIVATE_UNSET_FLAG (widget, GTK_REQUEST_NEEDED);
     }
 }
 
@@ -520,6 +606,9 @@ compute_dimension (GtkWidget        *widget,
 
   add_widget_to_closure (widget, mode, &groups, &widgets);
 
+  g_slist_foreach (widgets, (GFunc)mark_unvisited, NULL);
+  g_slist_foreach (groups, (GFunc)mark_unvisited, NULL);
+  
   g_slist_foreach (widgets, (GFunc)g_object_ref, NULL);
   
   if (!groups)
@@ -543,9 +632,12 @@ compute_dimension (GtkWidget        *widget,
 
 	      gint dimension = compute_base_dimension (tmp_widget, mode);
 
-	      if (dimension > result)
-		result = dimension;
-	      
+	      if (GTK_WIDGET_VISIBLE (tmp_widget) || !group->ignore_hidden)
+		{
+		  if (dimension > result)
+		    result = dimension;
+		}
+
 	      tmp_list = tmp_list->next;
 	    }
 
@@ -571,7 +663,7 @@ compute_dimension (GtkWidget        *widget,
     }
 
   g_slist_foreach (widgets, (GFunc)g_object_unref, NULL);
-  
+
   g_slist_free (widgets);
   g_slist_free (groups);
 
@@ -588,6 +680,9 @@ get_dimension (GtkWidget        *widget,
 
   add_widget_to_closure (widget, mode, &groups, &widgets);
 
+  g_slist_foreach (widgets, (GFunc)mark_unvisited, NULL);
+  g_slist_foreach (groups, (GFunc)mark_unvisited, NULL);  
+
   if (!groups)
     {
       result = get_base_dimension (widget, mode);
@@ -699,3 +794,6 @@ _gtk_size_group_queue_resize (GtkWidget *widget)
 {
   queue_resize_on_widget (widget, TRUE);
 }
+
+#define __GTK_SIZE_GROUP_C__
+#include "gtkaliasdef.c"
