@@ -140,8 +140,6 @@ enum {
 
   LAST_ARG
 };
-static void gtk_text_tag_init         (GtkTextTag      *text_tag);
-static void gtk_text_tag_class_init   (GtkTextTagClass *klass);
 static void gtk_text_tag_finalize     (GObject         *object);
 static void gtk_text_tag_set_property (GObject         *object,
                                        guint            prop_id,
@@ -152,43 +150,15 @@ static void gtk_text_tag_get_property (GObject         *object,
                                        GValue          *value,
                                        GParamSpec      *pspec);
 
-static GObjectClass *parent_class = NULL;
 static guint signals[LAST_SIGNAL] = { 0 };
 
-GType
-gtk_text_tag_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (our_type == 0)
-    {
-      static const GTypeInfo our_info =
-      {
-        sizeof (GtkTextTagClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) gtk_text_tag_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (GtkTextTag),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) gtk_text_tag_init
-      };
-
-      our_type = g_type_register_static (G_TYPE_OBJECT, "GtkTextTag",
-                                         &our_info, 0);
-    }
-
-  return our_type;
-}
+G_DEFINE_TYPE (GtkTextTag, gtk_text_tag, G_TYPE_OBJECT);
 
 static void
 gtk_text_tag_class_init (GtkTextTagClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
-
   object_class->set_property = gtk_text_tag_set_property;
   object_class->get_property = gtk_text_tag_get_property;
   
@@ -270,7 +240,7 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                                                       P_("Text direction"),
                                                       P_("Text direction, e.g. right-to-left or left-to-right"),
                                                       GTK_TYPE_TEXT_DIRECTION,
-                                                      GTK_TEXT_DIR_LTR,
+                                                      GTK_TEXT_DIR_NONE,
                                                       GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
@@ -281,6 +251,14 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                                                          TRUE,
                                                          GTK_PARAM_READWRITE));
 
+  /**
+   * GtkTextTag:font:
+   *
+   * Font description as string, e.g. \"Sans Italic 12\". 
+   *
+   * Note that the initial value of this property depends on
+   * the internals of #PangoFontDescription.
+   */
   g_object_class_install_property (object_class,
                                    PROP_FONT,
                                    g_param_spec_string ("font",
@@ -296,7 +274,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                                                        P_("Font description as a PangoFontDescription struct"),
                                                        PANGO_TYPE_FONT_DESCRIPTION,
                                                        GTK_PARAM_READWRITE));
-
   
   g_object_class_install_property (object_class,
                                    PROP_FAMILY,
@@ -382,7 +359,17 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                                                       GTK_TYPE_JUSTIFICATION,
                                                       GTK_JUSTIFY_LEFT,
                                                       GTK_PARAM_READWRITE));
-  
+
+  /**
+   * GtkTextTag:language:
+   *
+   * The language this text is in, as an ISO code. Pango can use this as a 
+   * hint when rendering the text. If not set, an appropriate default will be 
+   * used.
+   *
+   * Note that the initial value of this property depends on the current
+   * locale, see also gtk_get_default_language().
+   */
   g_object_class_install_property (object_class,
                                    PROP_LANGUAGE,
                                    g_param_spec_string ("language",
@@ -665,7 +652,7 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                 P_("Whether this tag affects the paragraph background color"));
   
   signals[EVENT] =
-    g_signal_new ("event",
+    g_signal_new (I_("event"),
                   G_OBJECT_CLASS_TYPE (object_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkTextTagClass, event),
@@ -681,9 +668,6 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
 static void
 gtk_text_tag_init (GtkTextTag *text_tag)
 {
-  /* 0 is basically a fine way to initialize everything in the
-     entire struct */
-  
   text_tag->values = gtk_text_attributes_new ();
 }
 
@@ -726,7 +710,7 @@ gtk_text_tag_finalize (GObject *object)
   g_free (text_tag->name);
   text_tag->name = NULL;
 
-  (* G_OBJECT_CLASS (parent_class)->finalize) (object);
+  (* G_OBJECT_CLASS (gtk_text_tag_parent_class)->finalize) (object);
 }
 
 static void
@@ -1489,16 +1473,12 @@ gtk_text_tag_get_property (GObject      *object,
 
     case PROP_FONT:
         {
-          /* FIXME GValue imposes a totally gratuitous string copy
-           * here, we could just hand off string ownership
-           */
           gchar *str;
 
 	  gtk_text_tag_ensure_font (tag);
 	  
 	  str = pango_font_description_to_string (tag->values->font);
-          g_value_set_string (value, str);
-          g_free (str);
+          g_value_take_string (value, str);
         }
       break;
 
@@ -1942,7 +1922,9 @@ gtk_text_attributes_new (void)
   values->language = gtk_get_default_language ();
 
   values->font_scale = 1.0;
-  
+
+  values->editable = TRUE;
+      
   return values;
 }
 
@@ -1971,7 +1953,7 @@ gtk_text_attributes_get_type (void)
   static GType our_type = 0;
   
   if (our_type == 0)
-    our_type = g_boxed_type_register_static ("GtkTextAttributes",
+    our_type = g_boxed_type_register_static (I_("GtkTextAttributes"),
 					     (GBoxedCopyFunc) gtk_text_attributes_ref,
 					     (GBoxedFreeFunc) gtk_text_attributes_unref);
 
@@ -2044,7 +2026,7 @@ gtk_text_attributes_copy_values (GtkTextAttributes *src,
  *
  * Returns: the #GtkTextAttributes that were passed in
  **/
-GtkTextAttributes* 
+GtkTextAttributes *
 gtk_text_attributes_ref (GtkTextAttributes *values)
 {
   g_return_val_if_fail (values != NULL, NULL);
