@@ -30,10 +30,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <gmodule.h>
-#include "gtkalias.h"
 #include "gtkthemes.h"
 #include "gtkrc.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 typedef struct _GtkThemeEngineClass GtkThemeEngineClass;
 
@@ -77,7 +77,7 @@ gtk_theme_engine_load (GTypeModule *module)
   
   GTK_NOTE (MISC, g_message ("Loading Theme %s\n", engine_path));
        
-  engine->library = g_module_open (engine_path, 0);
+  engine->library = g_module_open (engine_path, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
   g_free(engine_path);
   if (!engine->library)
     {
@@ -150,7 +150,7 @@ gtk_theme_engine_get_type (void)
       };
 
       theme_engine_type =
-	g_type_register_static (G_TYPE_TYPE_MODULE, "GtkThemeEngine",
+	g_type_register_static (G_TYPE_TYPE_MODULE, I_("GtkThemeEngine"),
 				&theme_engine_info, 0);
     }
   
@@ -191,3 +191,6 @@ gtk_theme_engine_create_rc_style (GtkThemeEngine *engine)
 
   return engine->create_rc_style ();
 }
+
+#define __GTK_THEMES_C__
+#include "gtkaliasdef.c"
