@@ -24,14 +24,13 @@
 #include "gtktreestore.h"
 #include "gtktreedatalist.h"
 #include "gtktreednd.h"
+#include "gtkintl.h"
 #include "gtkalias.h"
 
 #define G_NODE(node) ((GNode *)node)
-#define GTK_TREE_STORE_IS_SORTED(tree) (GTK_TREE_STORE (tree)->sort_column_id != GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID)
-#define VALID_ITER(iter, tree_store) (iter!= NULL && iter->user_data != NULL && tree_store->stamp == iter->stamp)
+#define GTK_TREE_STORE_IS_SORTED(tree) (((GtkTreeStore*)(tree))->sort_column_id != GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID)
+#define VALID_ITER(iter, tree_store) ((iter)!= NULL && (iter)->user_data != NULL && ((GtkTreeStore*)(tree_store))->stamp == (iter)->stamp)
 
-static void         gtk_tree_store_init            (GtkTreeStore      *tree_store);
-static void         gtk_tree_store_class_init      (GtkTreeStoreClass *tree_store_class);
 static void         gtk_tree_store_tree_model_init (GtkTreeModelIface *iface);
 static void         gtk_tree_store_drag_source_init(GtkTreeDragSourceIface *iface);
 static void         gtk_tree_store_drag_dest_init  (GtkTreeDragDestIface   *iface);
@@ -97,7 +96,8 @@ static gboolean gtk_tree_store_row_drop_possible  (GtkTreeDragDest   *drag_dest,
 static void     gtk_tree_store_sort                    (GtkTreeStore           *tree_store);
 static void     gtk_tree_store_sort_iter_changed       (GtkTreeStore           *tree_store,
 							GtkTreeIter            *iter,
-							gint                    column);
+							gint                    column,
+							gboolean                emit_signal);
 static gboolean gtk_tree_store_get_sort_column_id      (GtkTreeSortable        *sortable,
 							gint                   *sort_column_id,
 							GtkSortType            *order);
@@ -123,9 +123,6 @@ static void     gtk_tree_store_move                    (GtkTreeStore           *
                                                         gboolean                before);
 
 
-static GObjectClass *parent_class = NULL;
-
-
 static inline void
 validate_tree (GtkTreeStore *tree_store)
 {
@@ -137,81 +134,21 @@ validate_tree (GtkTreeStore *tree_store)
     }
 }
 
-GType
-gtk_tree_store_get_type (void)
-{
-  static GType tree_store_type = 0;
-
-  if (!tree_store_type)
-    {
-      static const GTypeInfo tree_store_info =
-      {
-        sizeof (GtkTreeStoreClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-        (GClassInitFunc) gtk_tree_store_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-        sizeof (GtkTreeStore),
-	0,              /* n_preallocs */
-        (GInstanceInitFunc) gtk_tree_store_init
-      };
-
-      static const GInterfaceInfo tree_model_info =
-      {
-	(GInterfaceInitFunc) gtk_tree_store_tree_model_init,
-	NULL,
-	NULL
-      };
-
-      static const GInterfaceInfo drag_source_info =
-      {
-	(GInterfaceInitFunc) gtk_tree_store_drag_source_init,
-	NULL,
-	NULL
-      };
-
-      static const GInterfaceInfo drag_dest_info =
-      {
-	(GInterfaceInitFunc) gtk_tree_store_drag_dest_init,
-	NULL,
-	NULL
-      };
-
-      static const GInterfaceInfo sortable_info =
-      {
-	(GInterfaceInitFunc) gtk_tree_store_sortable_init,
-	NULL,
-	NULL
-      };
-
-      tree_store_type = g_type_register_static (G_TYPE_OBJECT, "GtkTreeStore",
-						&tree_store_info, 0);
-
-      g_type_add_interface_static (tree_store_type,
-				   GTK_TYPE_TREE_MODEL,
-				   &tree_model_info);
-      g_type_add_interface_static (tree_store_type,
-				   GTK_TYPE_TREE_DRAG_SOURCE,
-				   &drag_source_info);
-      g_type_add_interface_static (tree_store_type,
-				   GTK_TYPE_TREE_DRAG_DEST,
-				   &drag_dest_info);
-      g_type_add_interface_static (tree_store_type,
-				   GTK_TYPE_TREE_SORTABLE,
-				   &sortable_info);
-
-    }
-
-  return tree_store_type;
-}
+G_DEFINE_TYPE_WITH_CODE (GtkTreeStore, gtk_tree_store, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
+						gtk_tree_store_tree_model_init)
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_DRAG_SOURCE,
+						gtk_tree_store_drag_source_init)
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_DRAG_DEST,
+						gtk_tree_store_drag_dest_init)
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_SORTABLE,
+						gtk_tree_store_sortable_init));
 
 static void
 gtk_tree_store_class_init (GtkTreeStoreClass *class)
 {
   GObjectClass *object_class;
 
-  parent_class = g_type_class_peek_parent (class);
   object_class = (GObjectClass *) class;
 
   object_class->finalize = gtk_tree_store_finalize;
@@ -398,8 +335,6 @@ gtk_tree_store_set_n_columns (GtkTreeStore *tree_store,
 {
   GType *new_columns;
 
-  g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
-
   if (tree_store->n_columns == n_columns)
     return;
 
@@ -441,8 +376,6 @@ gtk_tree_store_set_column_type (GtkTreeStore *tree_store,
 				gint          column,
 				GType         type)
 {
-  g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
-  g_return_if_fail (column >=0 && column < tree_store->n_columns);
   if (!_gtk_tree_data_list_check_type (type))
     {
       g_warning ("%s: Invalid type %s passed to gtk_tree_store_new_with_types\n", G_STRLOC, g_type_name (type));
@@ -482,7 +415,7 @@ gtk_tree_store_finalize (GObject *object)
     }
 
   /* must chain up */
-  (* parent_class->finalize) (object);
+  G_OBJECT_CLASS (gtk_tree_store_parent_class)->finalize (object);
 }
 
 /* fulfill the GtkTreeModel requirements */
@@ -495,8 +428,6 @@ gtk_tree_store_finalize (GObject *object)
 static GtkTreeModelFlags
 gtk_tree_store_get_flags (GtkTreeModel *tree_model)
 {
-  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), 0);
-
   return GTK_TREE_MODEL_ITERS_PERSIST;
 }
 
@@ -505,8 +436,6 @@ gtk_tree_store_get_n_columns (GtkTreeModel *tree_model)
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) tree_model;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), 0);
-
   tree_store->columns_dirty = TRUE;
 
   return tree_store->n_columns;
@@ -518,9 +447,7 @@ gtk_tree_store_get_column_type (GtkTreeModel *tree_model,
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) tree_model;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), G_TYPE_INVALID);
-  g_return_val_if_fail (index < GTK_TREE_STORE (tree_model)->n_columns &&
-			index >= 0, G_TYPE_INVALID);
+  g_return_val_if_fail (index < tree_store->n_columns, G_TYPE_INVALID);
 
   tree_store->columns_dirty = TRUE;
 
@@ -537,8 +464,6 @@ gtk_tree_store_get_iter (GtkTreeModel *tree_model,
   gint *indices;
   gint depth, i;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_store), FALSE);
-
   tree_store->columns_dirty = TRUE;
 
   indices = gtk_tree_path_get_indices (path);
@@ -549,13 +474,13 @@ gtk_tree_store_get_iter (GtkTreeModel *tree_model,
   parent.stamp = tree_store->stamp;
   parent.user_data = tree_store->root;
 
-  if (! gtk_tree_model_iter_nth_child (tree_model, iter, &parent, indices[0]))
+  if (!gtk_tree_store_iter_nth_child (tree_model, iter, &parent, indices[0]))
     return FALSE;
 
   for (i = 1; i < depth; i++)
     {
       parent = *iter;
-      if (! gtk_tree_model_iter_nth_child (tree_model, iter, &parent, indices[i]))
+      if (!gtk_tree_store_iter_nth_child (tree_model, iter, &parent, indices[i]))
 	return FALSE;
     }
 
@@ -566,26 +491,25 @@ static GtkTreePath *
 gtk_tree_store_get_path (GtkTreeModel *tree_model,
 			 GtkTreeIter  *iter)
 {
+  GtkTreeStore *tree_store = (GtkTreeStore *) tree_model;
   GtkTreePath *retval;
   GNode *tmp_node;
   gint i = 0;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), NULL);
-  g_return_val_if_fail (iter != NULL, NULL);
   g_return_val_if_fail (iter->user_data != NULL, NULL);
-  g_return_val_if_fail (iter->stamp == GTK_TREE_STORE (tree_model)->stamp, NULL);
+  g_return_val_if_fail (iter->stamp == tree_store->stamp, NULL);
 
-  validate_tree ((GtkTreeStore*)tree_model);
+  validate_tree (tree_store);
 
   if (G_NODE (iter->user_data)->parent == NULL &&
-      G_NODE (iter->user_data) == GTK_TREE_STORE (tree_model)->root)
+      G_NODE (iter->user_data) == tree_store->root)
     return gtk_tree_path_new ();
   g_assert (G_NODE (iter->user_data)->parent != NULL);
 
-  if (G_NODE (iter->user_data)->parent == G_NODE (GTK_TREE_STORE (tree_model)->root))
+  if (G_NODE (iter->user_data)->parent == G_NODE (tree_store->root))
     {
       retval = gtk_tree_path_new ();
-      tmp_node = G_NODE (GTK_TREE_STORE (tree_model)->root)->children;
+      tmp_node = G_NODE (tree_store->root)->children;
     }
   else
     {
@@ -593,8 +517,7 @@ gtk_tree_store_get_path (GtkTreeModel *tree_model,
 
       tmp_iter.user_data = G_NODE (iter->user_data)->parent;
 
-      retval = gtk_tree_store_get_path (tree_model,
-					&tmp_iter);
+      retval = gtk_tree_store_get_path (tree_model, &tmp_iter);
       tmp_node = G_NODE (iter->user_data)->parent->children;
     }
 
@@ -634,13 +557,12 @@ gtk_tree_store_get_value (GtkTreeModel *tree_model,
 			  gint          column,
 			  GValue       *value)
 {
+  GtkTreeStore *tree_store = (GtkTreeStore *) tree_model;
   GtkTreeDataList *list;
   gint tmp_column = column;
 
-  g_return_if_fail (GTK_IS_TREE_STORE (tree_model));
-  g_return_if_fail (iter != NULL);
-  g_return_if_fail (iter->stamp == GTK_TREE_STORE (tree_model)->stamp);
-  g_return_if_fail (column < GTK_TREE_STORE (tree_model)->n_columns);
+  g_return_if_fail (column < tree_store->n_columns);
+  g_return_if_fail (VALID_ITER (iter, tree_store));
 
   list = G_NODE (iter->user_data)->data;
 
@@ -650,13 +572,13 @@ gtk_tree_store_get_value (GtkTreeModel *tree_model,
   if (list)
     {
       _gtk_tree_data_list_node_to_value (list,
-					 GTK_TREE_STORE (tree_model)->column_headers[column],
+					 tree_store->column_headers[column],
 					 value);
     }
   else
     {
       /* We want to return an initialized but empty (default) value */
-      g_value_init (value, GTK_TREE_STORE (tree_model)->column_headers[column]);
+      g_value_init (value, tree_store->column_headers[column]);
     }
 }
 
@@ -664,7 +586,6 @@ static gboolean
 gtk_tree_store_iter_next (GtkTreeModel  *tree_model,
 			  GtkTreeIter   *iter)
 {
-  g_return_val_if_fail (iter != NULL, FALSE);
   g_return_val_if_fail (iter->user_data != NULL, FALSE);
   g_return_val_if_fail (iter->stamp == GTK_TREE_STORE (tree_model)->stamp, FALSE);
 
@@ -682,19 +603,20 @@ gtk_tree_store_iter_children (GtkTreeModel *tree_model,
 			      GtkTreeIter  *iter,
 			      GtkTreeIter  *parent)
 {
+  GtkTreeStore *tree_store = (GtkTreeStore *) tree_model;
   GNode *children;
 
-  g_return_val_if_fail (parent == NULL || parent->user_data != NULL, FALSE);
-  g_return_val_if_fail (parent == NULL || parent->stamp == GTK_TREE_STORE (tree_model)->stamp, FALSE);
+  if (parent)
+    g_return_val_if_fail (VALID_ITER (parent, tree_store), FALSE);
 
   if (parent)
     children = G_NODE (parent->user_data)->children;
   else
-    children = G_NODE (GTK_TREE_STORE (tree_model)->root)->children;
+    children = G_NODE (tree_store->root)->children;
 
   if (children)
     {
-      iter->stamp = GTK_TREE_STORE (tree_model)->stamp;
+      iter->stamp = tree_store->stamp;
       iter->user_data = children;
       return TRUE;
     }
@@ -706,9 +628,8 @@ static gboolean
 gtk_tree_store_iter_has_child (GtkTreeModel *tree_model,
 			       GtkTreeIter  *iter)
 {
-  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), FALSE);
-  g_return_val_if_fail (iter->stamp == GTK_TREE_STORE (tree_model)->stamp, FALSE);
   g_return_val_if_fail (iter->user_data != NULL, FALSE);
+  g_return_val_if_fail (VALID_ITER (iter, tree_model), FALSE);
 
   return G_NODE (iter->user_data)->children != NULL;
 }
@@ -720,7 +641,6 @@ gtk_tree_store_iter_n_children (GtkTreeModel *tree_model,
   GNode *node;
   gint i = 0;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), 0);
   g_return_val_if_fail (iter == NULL || iter->user_data != NULL, 0);
 
   if (iter == NULL)
@@ -743,14 +663,14 @@ gtk_tree_store_iter_nth_child (GtkTreeModel *tree_model,
 			       GtkTreeIter  *parent,
 			       gint          n)
 {
+  GtkTreeStore *tree_store = (GtkTreeStore *) tree_model;
   GNode *parent_node;
   GNode *child;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), FALSE);
   g_return_val_if_fail (parent == NULL || parent->user_data != NULL, FALSE);
 
   if (parent == NULL)
-    parent_node = GTK_TREE_STORE (tree_model)->root;
+    parent_node = tree_store->root;
   else
     parent_node = parent->user_data;
 
@@ -759,7 +679,7 @@ gtk_tree_store_iter_nth_child (GtkTreeModel *tree_model,
   if (child)
     {
       iter->user_data = child;
-      iter->stamp = GTK_TREE_STORE (tree_model)->stamp;
+      iter->stamp = tree_store->stamp;
       return TRUE;
     }
   else
@@ -771,21 +691,20 @@ gtk_tree_store_iter_parent (GtkTreeModel *tree_model,
 			    GtkTreeIter  *iter,
 			    GtkTreeIter  *child)
 {
+  GtkTreeStore *tree_store = (GtkTreeStore *) tree_model;
   GNode *parent;
 
   g_return_val_if_fail (iter != NULL, FALSE);
-  g_return_val_if_fail (child != NULL, FALSE);
-  g_return_val_if_fail (child->user_data != NULL, FALSE);
-  g_return_val_if_fail (child->stamp == GTK_TREE_STORE (tree_model)->stamp, FALSE);
+  g_return_val_if_fail (VALID_ITER (child, tree_store), FALSE);
 
   parent = G_NODE (child->user_data)->parent;
 
   g_assert (parent != NULL);
 
-  if (parent != GTK_TREE_STORE (tree_model)->root)
+  if (parent != tree_store->root)
     {
       iter->user_data = parent;
-      iter->stamp = GTK_TREE_STORE (tree_model)->stamp;
+      iter->stamp = tree_store->stamp;
       return TRUE;
     }
   else
@@ -808,11 +727,6 @@ gtk_tree_store_real_set_value (GtkTreeStore *tree_store,
   gboolean converted = FALSE;
   gboolean retval = FALSE;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_store), FALSE);
-  g_return_val_if_fail (VALID_ITER (iter, tree_store), FALSE);
-  g_return_val_if_fail (column >= 0 && column < tree_store->n_columns, FALSE);
-  g_return_val_if_fail (G_IS_VALUE (value), FALSE);
-
   if (! g_type_is_a (G_VALUE_TYPE (value), tree_store->column_headers[column]))
     {
       if (! (g_value_type_compatible (G_VALUE_TYPE (value), tree_store->column_headers[column]) &&
@@ -850,7 +764,7 @@ gtk_tree_store_real_set_value (GtkTreeStore *tree_store,
 	  if (converted)
 	    g_value_unset (&real_value);
           if (sort && GTK_TREE_STORE_IS_SORTED (tree_store))
-            gtk_tree_store_sort_iter_changed (tree_store, iter, old_column);
+            gtk_tree_store_sort_iter_changed (tree_store, iter, old_column, TRUE);
 	  return retval;
 	}
 
@@ -888,7 +802,7 @@ gtk_tree_store_real_set_value (GtkTreeStore *tree_store,
     g_value_unset (&real_value);
 
   if (sort && GTK_TREE_STORE_IS_SORTED (tree_store))
-    gtk_tree_store_sort_iter_changed (tree_store, iter, old_column);
+    gtk_tree_store_sort_iter_changed (tree_store, iter, old_column, TRUE);
 
   return retval;
 }
@@ -920,35 +834,22 @@ gtk_tree_store_set_value (GtkTreeStore *tree_store,
     {
       GtkTreePath *path;
 
-      path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), iter);
+      path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), iter);
       gtk_tree_model_row_changed (GTK_TREE_MODEL (tree_store), path, iter);
       gtk_tree_path_free (path);
     }
 }
 
-/**
- * gtk_tree_store_set_valist:
- * @tree_store: A #GtkTreeStore
- * @iter: A valid #GtkTreeIter for the row being modified
- * @var_args: <type>va_list</type> of column/value pairs
- *
- * See gtk_tree_store_set(); this version takes a <type>va_list</type> for
- * use by language bindings.
- *
- **/
-void
-gtk_tree_store_set_valist (GtkTreeStore *tree_store,
-                           GtkTreeIter  *iter,
-                           va_list	var_args)
+static void
+gtk_tree_store_set_valist_internal (GtkTreeStore *tree_store,
+                                    GtkTreeIter  *iter,
+                                    gboolean     *emit_signal,
+                                    gboolean     *maybe_need_sort,
+                                    va_list       var_args)
 {
   gint column;
-  gboolean emit_signal = FALSE;
-  gboolean maybe_need_sort = FALSE;
   GtkTreeIterCompareFunc func = NULL;
 
-  g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
-  g_return_if_fail (VALID_ITER (iter, tree_store));
-
   column = va_arg (var_args, gint);
 
   if (GTK_TREE_STORE_IS_SORTED (tree_store))
@@ -969,7 +870,7 @@ gtk_tree_store_set_valist (GtkTreeStore *tree_store,
     }
 
   if (func != _gtk_tree_data_list_compare_func)
-    maybe_need_sort = TRUE;
+    *maybe_need_sort = TRUE;
 
   while (column != -1)
     {
@@ -995,29 +896,56 @@ gtk_tree_store_set_valist (GtkTreeStore *tree_store,
 	  break;
 	}
 
-      emit_signal = gtk_tree_store_real_set_value (tree_store,
-						   iter,
-						   column,
-						   &value,
-						   FALSE) || emit_signal;
+      *emit_signal = gtk_tree_store_real_set_value (tree_store,
+						    iter,
+						    column,
+						    &value,
+						    FALSE) || *emit_signal;
 
       if (func == _gtk_tree_data_list_compare_func &&
 	  column == tree_store->sort_column_id)
-	maybe_need_sort = TRUE;
+	*maybe_need_sort = TRUE;
 
       g_value_unset (&value);
 
       column = va_arg (var_args, gint);
     }
+}
+
+/**
+ * gtk_tree_store_set_valist:
+ * @tree_store: A #GtkTreeStore
+ * @iter: A valid #GtkTreeIter for the row being modified
+ * @var_args: <type>va_list</type> of column/value pairs
+ *
+ * See gtk_tree_store_set(); this version takes a <type>va_list</type> for
+ * use by language bindings.
+ *
+ **/
+void
+gtk_tree_store_set_valist (GtkTreeStore *tree_store,
+                           GtkTreeIter  *iter,
+                           va_list       var_args)
+{
+  gboolean emit_signal = FALSE;
+  gboolean maybe_need_sort = FALSE;
+
+  g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
+  g_return_if_fail (VALID_ITER (iter, tree_store));
+
+  gtk_tree_store_set_valist_internal (tree_store, iter,
+				      &emit_signal,
+				      &maybe_need_sort,
+				      var_args);
 
   if (maybe_need_sort && GTK_TREE_STORE_IS_SORTED (tree_store))
-    gtk_tree_store_sort_iter_changed (tree_store, iter, tree_store->sort_column_id);
+    gtk_tree_store_sort_iter_changed (tree_store, iter, tree_store->sort_column_id, TRUE);
 
   if (emit_signal)
     {
       GtkTreePath *path;
 
-      path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), iter);
+      path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), iter);
       gtk_tree_model_row_changed (GTK_TREE_MODEL (tree_store), path, iter);
       gtk_tree_path_free (path);
     }
@@ -1043,9 +971,6 @@ gtk_tree_store_set (GtkTreeStore *tree_store,
 {
   va_list var_args;
 
-  g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
-  g_return_if_fail (VALID_ITER (iter, tree_store));
-
   va_start (var_args, iter);
   gtk_tree_store_set_valist (tree_store, iter, var_args);
   va_end (var_args);
@@ -1256,7 +1181,7 @@ gtk_tree_store_insert_before (GtkTreeStore *tree_store,
 
   gtk_tree_path_free (path);
 
-  validate_tree ((GtkTreeStore*)tree_store);
+  validate_tree (tree_store);
 }
 
 /**
@@ -1337,7 +1262,196 @@ gtk_tree_store_insert_after (GtkTreeStore *tree_store,
 
   gtk_tree_path_free (path);
 
-  validate_tree ((GtkTreeStore*)tree_store);
+  validate_tree (tree_store);
+}
+
+/**
+ * gtk_tree_store_insert_with_values:
+ * @tree_store: A #GtkTreeStore
+ * @iter: An unset #GtkTreeIter to set the new row
+ * @parent: A valid #GtkTreeIter, or %NULL
+ * @position: position to insert the new row
+ * @Varargs: pairs of column number and value, terminated with -1
+ *
+ * Creates a new row at @position.  @iter will be changed to point to this
+ * new row.  If @position is larger than the number of rows on the list, then
+ * the new row will be appended to the list.  The row will be filled with
+ * the values given to this function.
+ *
+ * Calling
+ * <literal>gtk_tree_store_insert_with_values (tree_store, iter, position, ...)</literal>
+ * has the same effect as calling
+ * <informalexample><programlisting>
+ * gtk_tree_store_insert (tree_store, iter, position);
+ * gtk_tree_store_set (tree_store, iter, ...);
+ * </programlisting></informalexample>
+ * with the different that the former will only emit a row_inserted signal,
+ * while the latter will emit row_inserted, row_changed and if the tree store
+ * is sorted, rows_reordered.  Since emitting the rows_reordered signal
+ * repeatedly can affect the performance of the program,
+ * gtk_tree_store_insert_with_values() should generally be preferred when
+ * inserting rows in a sorted tree store.
+ *
+ * Since: 2.10
+ */
+void
+gtk_tree_store_insert_with_values (GtkTreeStore *tree_store,
+				   GtkTreeIter  *iter,
+				   GtkTreeIter  *parent,
+				   gint          position,
+				   ...)
+{
+  GtkTreePath *path;
+  GNode *parent_node;
+  GNode *new_node;
+  va_list var_args;
+  gboolean changed = FALSE;
+  gboolean maybe_need_sort = FALSE;
+
+  g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
+  g_return_if_fail (iter != NULL);
+  if (parent)
+    g_return_if_fail (VALID_ITER (parent, tree_store));
+
+  if (parent)
+    parent_node = parent->user_data;
+  else
+    parent_node = tree_store->root;
+
+  tree_store->columns_dirty = TRUE;
+
+  new_node = g_node_new (NULL);
+
+  iter->stamp = tree_store->stamp;
+  iter->user_data = new_node;
+  g_node_insert (parent_node, position, new_node);
+
+  va_start (var_args, position);
+  gtk_tree_store_set_valist_internal (tree_store, iter,
+				      &changed, &maybe_need_sort,
+				      var_args);
+  va_end (var_args);
+
+  if (maybe_need_sort && GTK_TREE_STORE_IS_SORTED (tree_store))
+    gtk_tree_store_sort_iter_changed (tree_store, iter, tree_store->sort_column_id, FALSE);
+
+  path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), iter);
+  gtk_tree_model_row_inserted (GTK_TREE_MODEL (tree_store), path, iter);
+
+  if (parent_node != tree_store->root)
+    {
+      if (new_node->prev == NULL && new_node->next == NULL)
+        {
+	  gtk_tree_path_up (path);
+	  gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (tree_store), path, parent);
+	}
+    }
+
+  gtk_tree_path_free (path);
+
+  validate_tree ((GtkTreeStore *)tree_store);
+}
+
+/**
+ * gtk_tree_store_insert_with_valuesv:
+ * @tree_store: A #GtkTreeStore
+ * @iter: An unset #GtkTreeIter to set the new row
+ * @parent: A valid #GtkTreeIter, or %NULL
+ * @position: position to insert the new row
+ * @columns: an array of column numbers
+ * @values: an array of GValues
+ * @n_values: the length of the @columns and @values arrays
+ *
+ * A variant of gtk_tree_store_insert_with_values() which takes
+ * the columns and values as two arrays, instead of varargs.  This
+ * function is mainly intended for language bindings.
+ *
+ * Since: 2.10
+ */
+void
+gtk_tree_store_insert_with_valuesv (GtkTreeStore *tree_store,
+				    GtkTreeIter  *iter,
+				    GtkTreeIter  *parent,
+				    gint          position,
+				    gint         *columns,
+				    GValue       *values,
+				    gint          n_values)
+{
+  GtkTreePath *path;
+  GNode *parent_node;
+  GNode *new_node;
+  gboolean changed = FALSE;
+  gboolean maybe_need_sort = FALSE;
+  GtkTreeIterCompareFunc func = NULL;
+  gint i;
+
+  g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
+  g_return_if_fail (iter != NULL);
+  if (parent)
+    g_return_if_fail (VALID_ITER (parent, tree_store));
+
+  if (parent)
+    parent_node = parent->user_data;
+  else
+    parent_node = tree_store->root;
+
+  tree_store->columns_dirty = TRUE;
+
+  new_node = g_node_new (NULL);
+
+  iter->stamp = tree_store->stamp;
+  iter->user_data = new_node;
+  g_node_insert (parent_node, position, new_node);
+
+  if (GTK_TREE_STORE_IS_SORTED (tree_store))
+    {
+      if (tree_store->sort_column_id != -1)
+        {
+	  GtkTreeDataSortHeader *header;
+	  header = _gtk_tree_data_list_get_header (tree_store->sort_list,
+						   tree_store->sort_column_id);
+	  g_return_if_fail (header != NULL);
+	  g_return_if_fail (header->func != NULL);
+	  func = header->func;
+	}
+      else
+        {
+	  func = tree_store->default_sort_func;
+        }
+    }
+
+  if (func != _gtk_tree_data_list_compare_func)
+    maybe_need_sort = TRUE;
+
+  for (i = 0; i < n_values; i++)
+    {
+      changed = gtk_tree_store_real_set_value (tree_store, iter,
+					       columns[i], &values[i],
+					       FALSE) || changed;
+
+      if (func == _gtk_tree_data_list_compare_func &&
+	  columns[i] == tree_store->sort_column_id)
+	maybe_need_sort = TRUE;
+    }
+
+  if (maybe_need_sort && GTK_TREE_STORE_IS_SORTED (tree_store))
+    gtk_tree_store_sort_iter_changed (tree_store, iter, tree_store->sort_column_id, FALSE);
+
+  path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), iter);
+  gtk_tree_model_row_inserted (GTK_TREE_MODEL (tree_store), path, iter);
+
+  if (parent_node != tree_store->root)
+    {
+      if (new_node->prev == NULL && new_node->next == NULL)
+        {
+	  gtk_tree_path_up (path);
+	  gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (tree_store), path, parent);
+	}
+    }
+
+  gtk_tree_path_free (path);
+
+  validate_tree ((GtkTreeStore *)tree_store);
 }
 
 /**
@@ -1395,7 +1509,7 @@ gtk_tree_store_prepend (GtkTreeStore *tree_store,
       gtk_tree_store_insert_after (tree_store, iter, parent, NULL);
     }
 
-  validate_tree ((GtkTreeStore*)tree_store);
+  validate_tree (tree_store);
 }
 
 /**
@@ -1419,7 +1533,6 @@ gtk_tree_store_append (GtkTreeStore *tree_store,
 
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
   g_return_if_fail (iter != NULL);
-
   if (parent != NULL)
     g_return_if_fail (VALID_ITER (parent, tree_store));
 
@@ -1454,7 +1567,7 @@ gtk_tree_store_append (GtkTreeStore *tree_store,
       gtk_tree_store_insert_before (tree_store, iter, parent, NULL);
     }
 
-  validate_tree ((GtkTreeStore*)tree_store);
+  validate_tree (tree_store);
 }
 
 /**
@@ -1504,7 +1617,7 @@ gtk_tree_store_iter_depth (GtkTreeStore *tree_store,
 
 /* simple ripoff from g_node_traverse_post_order */
 static gboolean
-gtk_tree_store_clear_traverse (GNode *node,
+gtk_tree_store_clear_traverse (GNode        *node,
 			       GtkTreeStore *store)
 {
   GtkTreeIter iter;
@@ -1634,9 +1747,7 @@ gtk_tree_store_drag_data_delete (GtkTreeDragSource *drag_source,
 {
   GtkTreeIter iter;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (drag_source), FALSE);
-
-  if (gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_source),
+  if (gtk_tree_store_get_iter (GTK_TREE_MODEL (drag_source),
                                &iter,
                                path))
     {
@@ -1655,8 +1766,6 @@ gtk_tree_store_drag_data_get (GtkTreeDragSource *drag_source,
                               GtkTreePath       *path,
                               GtkSelectionData  *selection_data)
 {
-  g_return_val_if_fail (GTK_IS_TREE_STORE (drag_source), FALSE);
-
   /* Note that we don't need to handle the GTK_TREE_MODEL_ROW
    * target, because the default handler does it for us, but
    * we do anyway for the convenience of someone maybe overriding the
@@ -1725,7 +1834,7 @@ recursive_node_copy (GtkTreeStore *tree_store,
 
   copy_node_data (tree_store, src_iter, dest_iter);
 
-  if (gtk_tree_model_iter_children (model,
+  if (gtk_tree_store_iter_children (model,
                                     &child,
                                     src_iter))
     {
@@ -1743,7 +1852,7 @@ recursive_node_copy (GtkTreeStore *tree_store,
 
           recursive_node_copy (tree_store, &child, &copy);
         }
-      while (gtk_tree_model_iter_next (model, &child));
+      while (gtk_tree_store_iter_next (model, &child));
     }
 }
 
@@ -1758,8 +1867,6 @@ gtk_tree_store_drag_data_received (GtkTreeDragDest   *drag_dest,
   GtkTreePath *src_path = NULL;
   gboolean retval = FALSE;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (drag_dest), FALSE);
-
   tree_model = GTK_TREE_MODEL (drag_dest);
   tree_store = GTK_TREE_STORE (drag_dest);
 
@@ -1775,7 +1882,7 @@ gtk_tree_store_drag_data_received (GtkTreeDragDest   *drag_dest,
       GtkTreeIter dest_iter;
       GtkTreePath *prev;
 
-      if (!gtk_tree_model_get_iter (src_model,
+      if (!gtk_tree_store_get_iter (src_model,
                                     &src_iter,
                                     src_path))
         {
@@ -1801,7 +1908,7 @@ gtk_tree_store_drag_data_received (GtkTreeDragDest   *drag_dest,
           if (gtk_tree_path_up (parent) &&
 	      gtk_tree_path_get_depth (parent) > 0)
             {
-              gtk_tree_model_get_iter (tree_model,
+              gtk_tree_store_get_iter (tree_model,
                                        &dest_parent,
                                        parent);
               dest_parent_p = &dest_parent;
@@ -1817,7 +1924,7 @@ gtk_tree_store_drag_data_received (GtkTreeDragDest   *drag_dest,
         }
       else
         {
-          if (gtk_tree_model_get_iter (tree_model, &dest_iter, prev))
+          if (gtk_tree_store_get_iter (tree_model, &dest_iter, prev))
             {
               GtkTreeIter tmp_iter = dest_iter;
 
@@ -1867,8 +1974,6 @@ gtk_tree_store_row_drop_possible (GtkTreeDragDest  *drag_dest,
   GtkTreePath *tmp = NULL;
   gboolean retval = FALSE;
   
-  g_return_val_if_fail (GTK_IS_TREE_STORE (drag_dest), FALSE);
-
   /* don't accept drops if the tree has been sorted */
   if (GTK_TREE_STORE_IS_SORTED (drag_dest))
     return FALSE;
@@ -1896,7 +2001,7 @@ gtk_tree_store_row_drop_possible (GtkTreeDragDest  *drag_dest,
 	tmp = gtk_tree_path_copy (dest_path);
 	gtk_tree_path_up (tmp);
 	
-	if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_dest),
+	if (!gtk_tree_store_get_iter (GTK_TREE_MODEL (drag_dest),
 				      &iter, tmp))
 	  goto out;
       }
@@ -2018,7 +2123,7 @@ gtk_tree_store_reorder (GtkTreeStore *tree_store,
 
   /* emit signal */
   if (parent)
-    path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), parent);
+    path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), parent);
   else
     path = gtk_tree_path_new ();
   gtk_tree_model_rows_reordered (GTK_TREE_MODEL (tree_store), path,
@@ -2060,8 +2165,8 @@ gtk_tree_store_swap (GtkTreeStore *tree_store,
   if (node_a == node_b)
     return;
 
-  path_a = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), a);
-  path_b = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), b);
+  path_a = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), a);
+  path_b = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), b);
 
   g_return_if_fail (path_a && path_b);
 
@@ -2091,7 +2196,7 @@ gtk_tree_store_swap (GtkTreeStore *tree_store,
           g_warning ("Given children don't have a common parent\n");
           return;
         }
-      gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_store), &parent,
+      gtk_tree_store_get_iter (GTK_TREE_MODEL (tree_store), &parent,
                                path_a);
       parent_node = G_NODE (parent.user_data);
     }
@@ -2198,8 +2303,8 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
   /* sanity checks */
   if (position)
     {
-      path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), iter);
-      pos_path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store),
+      path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), iter);
+      pos_path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store),
 	                                  position);
 
       /* if before:
@@ -2248,7 +2353,7 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
 
   if (!path)
     {
-      path = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), iter);
+      path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), iter);
       gtk_tree_path_up (path);
     }
 
@@ -2256,7 +2361,8 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
 
   if (depth)
     {
-      gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_store), &parent_iter, path);
+      gtk_tree_store_get_iter (GTK_TREE_MODEL (tree_store), 
+			       &parent_iter, path);
 
       parent = G_NODE (parent_iter.user_data);
     }
@@ -2274,7 +2380,8 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
       if (gtk_tree_path_get_indices (pos_path)[gtk_tree_path_get_depth (pos_path) - 1] > 0)
         {
           gtk_tree_path_prev (pos_path);
-          if (gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_store), &dst_a, pos_path))
+          if (gtk_tree_store_get_iter (GTK_TREE_MODEL (tree_store), 
+				       &dst_a, pos_path))
             a = G_NODE (dst_a.user_data);
           else
             a = NULL;
@@ -2302,7 +2409,7 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
       if (position)
         {
           gtk_tree_path_next (pos_path);
-          if (gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_store), &dst_b, pos_path))
+          if (gtk_tree_store_get_iter (GTK_TREE_MODEL (tree_store), &dst_b, pos_path))
              b = G_NODE (dst_b.user_data);
           else
              b = NULL;
@@ -2312,10 +2419,10 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
         {
 	  /* move after without position is prepending */
 	  if (depth)
-	    gtk_tree_model_iter_children (GTK_TREE_MODEL (tree_store), &dst_b,
+	    gtk_tree_store_iter_children (GTK_TREE_MODEL (tree_store), &dst_b,
 	                                  &parent_iter);
 	  else
-	    gtk_tree_model_iter_children (GTK_TREE_MODEL (tree_store), &dst_b,
+	    gtk_tree_store_iter_children (GTK_TREE_MODEL (tree_store), &dst_b,
 		                          NULL);
 
 	  b = G_NODE (dst_b.user_data);
@@ -2418,10 +2525,10 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
   else if (before)
     {
       if (depth)
-        new_pos = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (tree_store),
+        new_pos = gtk_tree_store_iter_n_children (GTK_TREE_MODEL (tree_store),
 	                                          &parent_iter) - 1;
       else
-	new_pos = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (tree_store),
+	new_pos = gtk_tree_store_iter_n_children (GTK_TREE_MODEL (tree_store),
 	                                          NULL) - 1;
     }
   else
@@ -2464,7 +2571,8 @@ gtk_tree_store_move (GtkTreeStore *tree_store,
 
   if (depth)
     {
-      tmppath = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_store), &parent_iter);
+      tmppath = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), 
+					 &parent_iter);
       gtk_tree_model_rows_reordered (GTK_TREE_MODEL (tree_store),
 				     tmppath, &parent_iter, order);
     }
@@ -2673,7 +2781,8 @@ gtk_tree_store_sort (GtkTreeStore *tree_store)
     {
       GtkTreeDataSortHeader *header = NULL;
 
-      header = _gtk_tree_data_list_get_header (tree_store->sort_list, tree_store->sort_column_id);
+      header = _gtk_tree_data_list_get_header (tree_store->sort_list, 
+					       tree_store->sort_column_id);
 
       /* We want to make sure that we have a function */
       g_return_if_fail (header != NULL);
@@ -2690,7 +2799,8 @@ gtk_tree_store_sort (GtkTreeStore *tree_store)
 static void
 gtk_tree_store_sort_iter_changed (GtkTreeStore *tree_store,
 				  GtkTreeIter  *iter,
-				  gint          column)
+				  gint          column,
+				  gboolean      emit_signal)
 {
   GNode *prev = NULL;
   GNode *next = NULL;
@@ -2837,6 +2947,9 @@ gtk_tree_store_sort_iter_changed (GtkTreeStore *tree_store,
       G_NODE (iter->user_data)->parent->children = G_NODE (iter->user_data);
     }
 
+  if (!emit_signal)
+    return;
+
   /* Emit the reordered signal. */
   length = g_node_n_children (node->parent);
   new_order = g_new (int, length);
@@ -2884,8 +2997,6 @@ gtk_tree_store_get_sort_column_id (GtkTreeSortable  *sortable,
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (sortable), FALSE);
-
   if (sort_column_id)
     * sort_column_id = tree_store->sort_column_id;
   if (order)
@@ -2905,8 +3016,6 @@ gtk_tree_store_set_sort_column_id (GtkTreeSortable  *sortable,
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
 
-  g_return_if_fail (GTK_IS_TREE_STORE (sortable));
-
   
   if ((tree_store->sort_column_id == sort_column_id) &&
       (tree_store->order == order))
@@ -2918,7 +3027,8 @@ gtk_tree_store_set_sort_column_id (GtkTreeSortable  *sortable,
 	{
 	  GtkTreeDataSortHeader *header = NULL;
 
-	  header = _gtk_tree_data_list_get_header (tree_store->sort_list, sort_column_id);
+	  header = _gtk_tree_data_list_get_header (tree_store->sort_list, 
+						   sort_column_id);
 
 	  /* We want to make sure that we have a function */
 	  g_return_if_fail (header != NULL);
@@ -2946,42 +3056,10 @@ gtk_tree_store_set_sort_func (GtkTreeSortable        *sortable,
 			      GtkDestroyNotify        destroy)
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
-  GtkTreeDataSortHeader *header = NULL;
-  GList *list;
 
-  g_return_if_fail (GTK_IS_TREE_STORE (sortable));
-  g_return_if_fail (func != NULL);
-
-  for (list = tree_store->sort_list; list; list = list->next)
-    {
-      GtkTreeDataSortHeader *list_header;
-
-      list_header = (GtkTreeDataSortHeader*) list->data;
-      if (list_header->sort_column_id == sort_column_id)
-	{
-	  header = list_header;
-	  break;
-	}
-    }
-
-  if (header == NULL)
-    {
-      header = g_new0 (GtkTreeDataSortHeader, 1);
-      header->sort_column_id = sort_column_id;
-      tree_store->sort_list = g_list_append (tree_store->sort_list, header);
-    }
-
-  if (header->destroy)
-    {
-      GtkDestroyNotify d = header->destroy;
-
-      header->destroy = NULL;
-      d (header->data);
-    }
-
-  header->func = func;
-  header->data = data;
-  header->destroy = destroy;
+  tree_store->sort_list = _gtk_tree_data_list_set_header (tree_store->sort_list, 
+							  sort_column_id, 
+							  func, data, destroy);
 
   if (tree_store->sort_column_id == sort_column_id)
     gtk_tree_store_sort (tree_store);
@@ -2995,8 +3073,6 @@ gtk_tree_store_set_default_sort_func (GtkTreeSortable        *sortable,
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
 
-  g_return_if_fail (GTK_IS_TREE_STORE (sortable));
-
   if (tree_store->default_sort_destroy)
     {
       GtkDestroyNotify d = tree_store->default_sort_destroy;
@@ -3018,8 +3094,6 @@ gtk_tree_store_has_default_sort_func (GtkTreeSortable *sortable)
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
 
-  g_return_val_if_fail (GTK_IS_TREE_STORE (sortable), FALSE);
-
   return (tree_store->default_sort_func != NULL);
 }
 
