@@ -27,8 +27,9 @@
 #include <config.h>
 #include <math.h>
 #include <string.h>
-#include "gtkalias.h"
 #include "gtkvruler.h"
+#include "gtkintl.h"
+#include "gtkalias.h"
 
 #include <glib/gprintf.h>
 
@@ -41,40 +42,12 @@
 #define ROUND(x) ((int) ((x) + 0.5))
 
 
-static void gtk_vruler_class_init    (GtkVRulerClass *klass);
-static void gtk_vruler_init          (GtkVRuler      *vruler);
 static gint gtk_vruler_motion_notify (GtkWidget      *widget,
 				      GdkEventMotion *event);
 static void gtk_vruler_draw_ticks    (GtkRuler       *ruler);
 static void gtk_vruler_draw_pos      (GtkRuler       *ruler);
 
-
-GType
-gtk_vruler_get_type (void)
-{
-  static GType vruler_type = 0;
-
-  if (!vruler_type)
-    {
-      static const GTypeInfo vruler_info =
-      {
-	sizeof (GtkVRulerClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_vruler_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkVRuler),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_vruler_init,
-      };
-
-      vruler_type = g_type_register_static (GTK_TYPE_RULER, "GtkVRuler",
-					    &vruler_info, 0);
-    }
-
-  return vruler_type;
-}
+G_DEFINE_TYPE (GtkVRuler, gtk_vruler, GTK_TYPE_RULER);
 
 static void
 gtk_vruler_class_init (GtkVRulerClass *klass)
@@ -136,7 +109,7 @@ static void
 gtk_vruler_draw_ticks (GtkRuler *ruler)
 {
   GtkWidget *widget;
-  GdkGC *gc, *bg_gc;
+  cairo_t *cr;
   gint i, j;
   gint width, height;
   gint xthickness;
@@ -160,9 +133,6 @@ gtk_vruler_draw_ticks (GtkRuler *ruler)
 
   widget = GTK_WIDGET (ruler);
 
-  gc = widget->style->fg_gc[GTK_STATE_NORMAL];
-  bg_gc = widget->style->bg_gc[GTK_STATE_NORMAL];
-
   xthickness = widget->style->xthickness;
   ythickness = widget->style->ythickness;
 
@@ -181,17 +151,21 @@ gtk_vruler_draw_ticks (GtkRuler *ruler)
 		 0, 0, 
 		  widget->allocation.width, widget->allocation.height);
   
-  gdk_draw_line (ruler->backing_store, gc,
-		 height + xthickness,
-		 ythickness,
-		 height + xthickness,
-		 widget->allocation.height - ythickness);
-  
+  cr = gdk_cairo_create (ruler->backing_store);
+  gdk_cairo_set_source_color (cr, &widget->style->fg[widget->state]);
+ 
+  cairo_rectangle (cr, 
+		   height + xthickness,
+		   ythickness,
+		   1,
+		   widget->allocation.height - 2 * ythickness);
+
   upper = ruler->upper / ruler->metric->pixels_per_unit;
   lower = ruler->lower / ruler->metric->pixels_per_unit;
 
   if ((upper - lower) == 0)
-    return;
+    goto out;
+
   increment = (gdouble) width / (upper - lower);
 
   /* determine the scale
@@ -242,9 +216,9 @@ gtk_vruler_draw_ticks (GtkRuler *ruler)
 	{
 	  pos = ROUND ((cur - lower) * increment);
 
-	  gdk_draw_line (ruler->backing_store, gc,
-			 height + xthickness - length, pos,
-			 height + xthickness, pos);
+	  cairo_rectangle (cr, 
+			   height + xthickness - length, pos,
+			   length,                       1);
 
 	  /* draw label */
 	  if (i == 0)
@@ -272,6 +246,10 @@ gtk_vruler_draw_ticks (GtkRuler *ruler)
 	}
     }
 
+  cairo_fill (cr);
+out:
+  cairo_destroy (cr);
+
   g_object_unref (layout);
 }
 
@@ -279,9 +257,7 @@ gtk_vruler_draw_ticks (GtkRuler *ruler)
 static void
 gtk_vruler_draw_pos (GtkRuler *ruler)
 {
-  GtkWidget *widget;
-  GdkGC *gc;
-  int i;
+  GtkWidget *widget = GTK_WIDGET (ruler);
   gint x, y;
   gint width, height;
   gint bs_width, bs_height;
@@ -291,24 +267,23 @@ gtk_vruler_draw_pos (GtkRuler *ruler)
 
   if (GTK_WIDGET_DRAWABLE (ruler))
     {
-      widget = GTK_WIDGET (ruler);
-
-      gc = widget->style->fg_gc[GTK_STATE_NORMAL];
       xthickness = widget->style->xthickness;
       ythickness = widget->style->ythickness;
       width = widget->allocation.width - xthickness * 2;
       height = widget->allocation.height;
 
-      bs_height = width / 2;
+      bs_height = width / 2 + 2;
       bs_height |= 1;  /* make sure it's odd */
       bs_width = bs_height / 2 + 1;
 
       if ((bs_width > 0) && (bs_height > 0))
 	{
+	  cairo_t *cr = gdk_cairo_create (widget->window);
+      
 	  /*  If a backing store exists, restore the ruler  */
-	  if (ruler->backing_store && ruler->non_gr_exp_gc)
-	    gdk_draw_drawable (ruler->widget.window,
-			       ruler->non_gr_exp_gc,
+	  if (ruler->backing_store)
+	    gdk_draw_drawable (widget->window,
+			       widget->style->black_gc,
 			       ruler->backing_store,
 			       ruler->xsrc, ruler->ysrc,
 			       ruler->xsrc, ruler->ysrc,
@@ -318,14 +293,21 @@ gtk_vruler_draw_pos (GtkRuler *ruler)
 
 	  x = (width + bs_width) / 2 + xthickness;
 	  y = ROUND ((ruler->position - ruler->lower) * increment) + (ythickness - bs_height) / 2 - 1;
+	  
+	  gdk_cairo_set_source_color (cr, &widget->style->fg[widget->state]);
 
-	  for (i = 0; i < bs_width; i++)
-	    gdk_draw_line (widget->window, gc,
-			   x + i, y + i,
-			   x + i, y + bs_height - 1 - i);
+	  cairo_move_to (cr, x,            y);
+	  cairo_line_to (cr, x + bs_width, y + bs_height / 2.);
+	  cairo_line_to (cr, x,            y + bs_height);
+	  cairo_fill (cr);
+
+	  cairo_destroy (cr);
 
 	  ruler->xsrc = x;
 	  ruler->ysrc = y;
 	}
     }
 }
+
+#define __GTK_VRULER_C__
+#include "gtkaliasdef.c"
